@@ -51,6 +51,8 @@
 #include "lib/backend.h"
 #include "lib/extension.h"
 #include "lib/assistant.h"
+#include "lib/panelpluginhandler.h"
+#include "lib/panelplugin.h"
 
 #include "settings.h"
 
@@ -66,6 +68,9 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QStringLis
     m_initProgressDlg=0;
     m_statusBarBlocked=false;
 
+    m_panelHandler=new Cantor::PanelPluginHandler(this);
+    connect(m_panelHandler, SIGNAL(pluginsChanged()), this, SLOT(pluginsChanged()));
+
     kDebug()<<"Created a CantorPart";
     QString backendName;
     if(args.isEmpty())
@@ -95,6 +100,8 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QStringLis
     setWidget(m_worksheet);
 
     // create our actions
+    m_worksheet->createActions( actionCollection() );
+
     KStandardAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
     m_save = KStandardAction::save(this, SLOT(save()), actionCollection());
 
@@ -143,10 +150,25 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QStringLis
     actionCollection()->addAction("evaluate_current",  evaluateCurrent);
     connect(evaluateCurrent, SIGNAL(triggered()), m_worksheet, SLOT(evaluateCurrentEntry()));
 
-    KAction* insertEntry=new KAction(i18n("Insert Entry"), actionCollection());
-    insertEntry->setShortcut(Qt::CTRL + Qt::Key_Return);
-    actionCollection()->addAction("insert_entry",  insertEntry);
-    connect(insertEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertEntry()));
+    KAction* insertCommandEntry=new KAction(i18n("Insert Command Entry"), actionCollection());
+    insertCommandEntry->setShortcut(Qt::CTRL + Qt::Key_Return);
+    actionCollection()->addAction("insert_command_entry",  insertCommandEntry);
+    connect(insertCommandEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertCommandEntry()));
+
+    KAction* insertTextEntry=new KAction(i18n("Insert Text Entry"), actionCollection());
+    //insertEntry->setShortcut(Qt::CTRL + Qt::Key_Return);
+    actionCollection()->addAction("insert_text_entry",  insertTextEntry);
+    connect(insertTextEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertTextEntry()));
+
+    KAction* insertCommandEntryBefore=new KAction(i18n("Insert Command Entry Before"), actionCollection());
+    //insertCommandEntryBefore->setShortcut(Qt::CTRL + Qt::Key_Return);
+    actionCollection()->addAction("insert_command_entry_before",  insertCommandEntryBefore);
+    connect(insertCommandEntryBefore, SIGNAL(triggered()), m_worksheet, SLOT(insertCommandEntryBefore()));
+
+    KAction* insertTextEntryBefore=new KAction(i18n("Insert Text Entry Before"), actionCollection());
+    //insertTextEntryBefore->setShortcut(Qt::CTRL + Qt::Key_Return);
+    actionCollection()->addAction("insert_text_entry_before",  insertTextEntryBefore);
+    connect(insertTextEntryBefore, SIGNAL(triggered()), m_worksheet, SLOT(insertTextEntryBefore()));
 
     KAction* removeCurrent=new KAction(i18n("Remove current Entry"), actionCollection());
     removeCurrent->setShortcut(Qt::ShiftModifier + Qt::Key_Delete);
@@ -174,7 +196,6 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QStringLis
     showCompletion->setShortcut(Qt::Key_Tab); //Qt::CTRL + Qt::Key_Space);
     connect(showCompletion, SIGNAL(triggered()), m_worksheet, SLOT(showCompletion()));
 
-
     // set our XML-UI resource file
     setXMLFile("cantor_part.rc");
 
@@ -366,6 +387,7 @@ void CantorPart::worksheetSessionChanged()
     connect(m_worksheet->session(), SIGNAL(error(const QString&)), this, SLOT(showSessionError(const QString&)));
 
     loadAssistants();
+    m_panelHandler->setSession(m_worksheet->session());
     adjustGuiToSession();
 
     if(!m_initProgressDlg)
@@ -378,7 +400,9 @@ void CantorPart::worksheetSessionChanged()
 
 void CantorPart::initialized()
 {
+    m_worksheet->appendCommandEntry();
     m_worksheet->setEnabled(true);
+    m_worksheet->setFocus();
     setStatusMessage(i18n("Initialization complete"));
 
     if(m_initProgressDlg)
@@ -420,6 +444,14 @@ void CantorPart::updateCaption()
     emit setCaption(i18n("%1: %2", m_worksheet->session()->backend()->name(), filename));
 }
 
+void CantorPart::pluginsChanged()
+{
+    foreach(Cantor::PanelPlugin* plugin, m_panelHandler->plugins())
+    {
+        connect(plugin, SIGNAL(requestRunCommand(QString)), this, SLOT(runCommand(QString)));
+    }
+}
+
 void CantorPart::loadAssistants()
 {
     kDebug()<<"loading assistants...";
@@ -474,7 +506,12 @@ void CantorPart::runAssistant()
     QStringList cmds=a->run(widget());
     kDebug()<<cmds;
     if(!cmds.isEmpty())
-        m_worksheet->appendEntry(cmds.join("\n"));
+        runCommand(cmds.join("\n"));
+}
+
+void CantorPart::runCommand(const QString& cmd)
+{
+    m_worksheet->appendCommandEntry(cmd);
 }
 
 void CantorPart::adjustGuiToSession()
@@ -573,7 +610,7 @@ void CantorPart::runScript(const QString& file)
     }
 
     Cantor::ScriptExtension* scriptE=dynamic_cast<Cantor::ScriptExtension*>(backend->extension("ScriptExtension"));
-    m_worksheet->appendEntry(scriptE->runExternalScript(file));
+    m_worksheet->appendCommandEntry(scriptE->runExternalScript(file));
 }
 
 void CantorPart::blockStatusBar()
