@@ -29,53 +29,56 @@
 #include "search.h"
 
 SearchWidget::SearchWidget( QWidget *parent )
-	: QWidget( parent ), m_timer( 0 )
+        : QWidget( parent ), m_timer( 0 )
 {
-	QHBoxLayout *mainlay = new QHBoxLayout( this );
-	mainlay->setMargin( 2 );
-	mainlay->setSpacing( 5 );
+    QHBoxLayout *mainlay = new QHBoxLayout( this );
+    mainlay->setMargin( 2 );
+    mainlay->setSpacing( 5 );
 
-	mainlay->addWidget( new QLabel( i18n( "Search:" ), this ) );
+    mainlay->addWidget( new QLabel( i18n( "Search:" ), this ) );
 
-	m_searchLine = new KLineEdit( this );
-	m_searchLine->setClearButtonShown(true);
+    m_searchLine = new KLineEdit( this );
+    m_searchLine->setClearButtonShown(true);
     m_searchLine->setTrapReturnKey(true);
-	connect( m_searchLine, SIGNAL( textChanged( const QString& ) ),
-	         this, SLOT( searchTextChanged( const QString& ) ) );
+    connect( m_searchLine, SIGNAL( textChanged( const QString& ) ),
+             this, SLOT( searchTextChanged( const QString& ) ) );
     connect( m_searchLine, SIGNAL( returnPressed() ),
              this, SLOT( slotReturnPressed() ) );
-	mainlay->addWidget( m_searchLine );
+    mainlay->addWidget( m_searchLine );
 }
 
 SearchWidget::~SearchWidget()
 {
+  delete m_searchLine;
+  delete m_timer;
 }
 
 void SearchWidget::giveFocus()
 {
-	m_searchLine->setFocus( Qt::MouseFocusReason );
-	m_searchLine->setCursorPosition( m_searchLine->text().length() );
+    m_searchLine->setFocus( Qt::MouseFocusReason );
+    m_searchLine->setCursorPosition( m_searchLine->text().length() );
 }
 
 void SearchWidget::appendSearchText( const QString& text )
 {
-	m_searchLine->setText( m_searchLine->text() + text );
+  // FIXME Reproducable crash when setting or getting text. Worked in old kalzium.
+//     m_searchLine->setText(m_searchLine->text() + text);
 }
 
 void SearchWidget::searchTextChanged( const QString& )
 {
-	if ( m_timer )
-	{
-		m_timer->stop();
-	}
-	else
-	{
-		m_timer = new QTimer( this );
-		m_timer->setSingleShot( true );
-		connect( m_timer, SIGNAL( timeout() ), this, SLOT( doSearch() ) );
-	}
-	// 1/3 of second should be ok
-	m_timer->start( 333 );
+    if ( m_timer )
+    {
+        m_timer->stop();
+    }
+    else
+    {
+        m_timer = new QTimer( this );
+        m_timer->setSingleShot( true );
+        connect( m_timer, SIGNAL( timeout() ), this, SLOT( doSearch() ) );
+    }
+    // 1/3 of second should be ok
+    m_timer->start( 333 );
 }
 
 void SearchWidget::slotReturnPressed()
@@ -89,14 +92,14 @@ void SearchWidget::slotReturnPressed()
 
 void SearchWidget::doSearch()
 {
-	Search *s = KalziumDataObject::instance()->search();
-	if ( !s ) return;
+    Search *s = KalziumDataObject::instance()->search();
+    if ( !s ) return;
 
-	QString txt = m_searchLine->text();
-	if ( txt.length() > 0 )
-		s->doSearch( txt, Search::SearchAll );
-	else
-		s->resetSearch();
+    QString txt = m_searchLine->text();
+    if ( txt.length() > 0 )
+        s->doSearch( txt, Search::SearchAll );
+    else
+        s->resetSearch();
 }
 
 #include "searchwidget.moc"
