@@ -171,7 +171,7 @@ void primenumber::display_all()
 /* finds next prime number and adds it to the vector */
 void primenumber::find_next()
 {
-	/* our new prime number, must be bigger then the last one */
+	/* our new prime number, must be bigger than the last one */
 	unsigned int new_prim = *(prim_vector.end() - 1);
 
 	do
@@ -180,7 +180,7 @@ void primenumber::find_next()
 		new_prim += 2;
 		/* loop as long as we find a divisor for the new number */
 		for (current_pos = prim_vector.begin(); current_pos != prim_vector.end();
-		        current_pos++)
+		        ++current_pos)
 			if ((new_prim % *current_pos == 0) || (new_prim < *current_pos))
 				break;
 
