@@ -28,8 +28,6 @@
 #include <qpushbutton.h>
 
 //Added by qt3to4:
-#include <QVBoxLayout>
-#include <QHBoxLayout>
 #include <QGridLayout>
 
 #include <klocale.h>
