@@ -30,10 +30,10 @@ class QRect;
 class QSvgRenderer;
 class KHMTheme;
 
-#include <qimage.h>
-#include <qpixmap.h>
+#include <QImage>
+#include <QPixmap>
 
-#include <klineedit.h>
+#include <KLineEdit>
 
 #include <krandomsequence.h>
 #include <keduvocdocument.h>
@@ -70,7 +70,7 @@ public:
 
     /// Enter a letter into the input widget.
     void  enterLetter(QString letter) { m_letterInput->setText(letter); }
-
+    
     int winCount;
     int lossCount;
 
@@ -105,7 +105,6 @@ private slots:
     /// popup informing about this is closed.
     void enableUserInput();
 
-
  private:
     // Painting
 
@@ -138,6 +137,7 @@ private slots:
      */
     QString stripAccents(const QString & original);
 
+    bool m_showhint;
     bool m_winner;
     bool m_loser;
     int m_bgfill;
