@@ -12,8 +12,8 @@
 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+// 02110-1301, USA.
 
 #ifndef KIG_MISC_CUBIC_COMMON_H
 #define KIG_MISC_CUBIC_COMMON_H
@@ -24,15 +24,27 @@ class Transformation;
 
 /**
  * This class represents an equation of a cubic in the form
- * "a_{ijk} x_i x_j x_k = 0" (in homogeneous coordinates, i,j,k = 0,1,2),
- * i <= j <= k.
+ * \f$ a_{ijk} x_i x_j x_k = 0 \f$ (in homogeneous coordinates,
+ * \f$ i,j,k = 0,1,2 \f$), \f$ i <= j <= k \f$.
  * The coefficients are stored in lessicografic order.
  */
 class CubicCartesianData
 {
 public:
   double coeffs[10];
+  /**
+   * \ifnot creating-python-scripting-doc
+   * \brief Default Constructor
+   *
+   * Constructs a new CubicCartesianData, with all the coeffs
+   * initialized to 0.
+   * \endif
+   */
   explicit CubicCartesianData();
+  /**
+   * Constructor. Construct a new CubicCartesianData, with the given
+   * values as coeffs.
+   */
   CubicCartesianData( double a000, double a001, double a002,
                       double a011, double a012, double a022,
                       double a111, double a112, double a122,
@@ -48,11 +60,32 @@ public:
       coeffs[7] = a112;
       coeffs[8] = a122;
       coeffs[9] = a222;
-    };
+    }
   CubicCartesianData( const double incoeffs[10] );
 
+  /**
+   * Create an invalid CubicCartesianData. This is a special state of a
+   * CubicCartesianData that signals that something went wrong..
+   *
+   * \see CubicCartesianData::valid
+   *
+   * \internal We represent an invalid CubicCartesianData by setting all
+   * the coeffs to positive or negative infinity. This is handy, since
+   * it doesn't require us to adapt most of the functions, it doesn't
+   * need extra space, and most of the times that we should get an
+   * invalid CubicCartesianData, we get one automatically..
+   */
   static CubicCartesianData invalidData();
+  /**
+   * Return whether this is a valid CubicCartesianData.
+   *
+   * \see CubicCartesianData::invalidData
+   */
   bool valid() const;
+  /*
+   * normalize mdata with infinity norm of the coefficients = 1
+   */
+  void normalize();
 };
 
 bool operator==( const CubicCartesianData& lhs, const CubicCartesianData& rhs );
@@ -63,7 +96,6 @@ bool operator==( const CubicCartesianData& lhs, const CubicCartesianData& rhs );
  * least 2 point.  If there are less than 9, than the coefficients
  * will be chosen to 1.0 if possible
  */
-
 const CubicCartesianData calcCubicThroughPoints (
     const std::vector<Coordinate>& points );
 
