@@ -54,7 +54,7 @@ public:
      * Creates an angle whose value in Degrees is equal to the argument.
      * @param x angle expressed as a floating-point number (in degrees)
      */
-    dms( const double &x ) { setD( x ); }
+    explicit dms( const double &x ) : D(x) {}
 
     /**@short Construct an angle from a string representation.
      *
@@ -95,127 +95,76 @@ public:
     inline const double& Degrees() const { return D; }
 
     /**@return integer hours portion of the angle
-    	*@note an angle can be measured in degrees/arcminutes/arcseconds
-    	*or hours/minutes/seconds.  An hour is equal to 15 degrees.
-    	*/
+     * @note an angle can be measured in degrees/arcminutes/arcseconds
+     * or hours/minutes/seconds.  An hour is equal to 15 degrees.
+     */
     inline int hour() const { return int( reduce().Degrees()/15.0 ); }
 
     /**@return integer minutes portion of the angle
-    	*@note a minute is 1/60 hour (not the same as an arcminute)
-    	*/
+     * @note a minute is 1/60 hour (not the same as an arcminute)
+     */
     int minute() const;
 
     /**@return integer seconds portion of the angle
-    	*@note a second is 1/3600 hour (not the same as an arcsecond)
-    	*/
+     * @note a second is 1/3600 hour (not the same as an arcsecond)
+     */
     int second() const;
 
     /**@return integer milliseconds portion of the angle
-    	*@note a millisecond is 1/1000 second (not the same as a milliarcsecond)
-    	*/
+     * @note a millisecond is 1/1000 second (not the same as a milliarcsecond)
+     */
     int msecond() const;
 
     /**@return angle in hours expressed as a double.
-    	*@note an angle can be measured in degrees/arcminutes/arcseconds
-    	*or hours/minutes/seconds.  An hour is equal to 15 degrees.
-    	*/
+     * @note an angle can be measured in degrees/arcminutes/arcseconds
+     * or hours/minutes/seconds.  An hour is equal to 15 degrees.
+     */
     inline double Hours() const { return reduce().Degrees()/15.0; }
 
-    /**Sets integer degrees portion of angle, leaving the arcminute and
-    	*arcsecond values intact.
-    	*@param d new integer degrees value
-    	*/
-    inline void setDeg( const int &d ) { setD( d, arcmin(), arcsec() ); }
-
-    /**Sets integer arcminutes portion of angle, leaving the degrees
-    	*and arcsecond values intact.
-    	*@param m new integer arcminutes value
-    	*/
-    inline void setArcMin( const int &m ) { setD( degree(), m, arcsec() ); }
-
-    /**Sets integer arcseconds portion of angle, leaving the degrees
-    	*and arcminute values intact.
-    	*@param s new integer arcseconds value
-    	*/
-    inline void setArcSec( const int &s ) { setD( degree(), arcmin(), s ); }
-
     /**Sets floating-point value of angle, in degrees.
-    	*@param x new angle (double)
-    	*/
-    void setD( const double &x );
+     * @param x new angle (double)
+     */
+    void setD( const double &x ) { D = x; }
 
     /**@short Sets floating-point value of angle, in degrees.
-    	*
-    	*This is an overloaded member function; it behaves essentially
-    	*like the above function.  The floating-point value of the angle
-    	*(D) is determined from the following formulae:
-    	*
-    	*\f$ fabs(D) = fabs(d) + \frac{(m + (s/60))}{60} \f$
-    	*\f$ sgn(D) = sgn(d) \f$
-    	*
-    	*@param d integer degrees portion of angle
-    	*@param m integer arcminutes portion of angle
-    	*@param s integer arcseconds portion of angle
-    	*@param ms integer arcseconds portion of angle
-    	*/
+     * 
+     * This is an overloaded member function; it behaves essentially
+     * like the above function.  The floating-point value of the angle
+     * (D) is determined from the following formulae:
+     * 
+     * \f $ fabs(D) = fabs(d) + \frac{(m + (s/60))}{60} \f$
+     * \f$ sgn(D) = sgn(d) \f$
+     * 
+     * @param d integer degrees portion of angle
+     * @param m integer arcminutes portion of angle
+     * @param s integer arcseconds portion of angle
+     * @param ms integer arcseconds portion of angle
+     */
     void setD( const int &d, const int &m, const int &s, const int &ms=0 );
 
-    /**Sets integer hours portion of angle, leaving the minutes and
-    	*seconds values intact.
-    	*@param h new integer hours value
-    	*@sa setH() setDeg()
-    	*/
-    void setHour( const int &h ) { setH( h, minute(), second() ); }
-
-    /**Sets integer minutes portion of angle, leaving the hours and
-    	*seconds values intact.
-    	*@param m new integer minutes value
-    	*@sa setArcMin()
-    	*/
-    void setHMin( const int &m ) { setH( hour(), m, second() ); }
-
-    /**Sets integer seconds portion of angle, leaving the hours and
-    	*minutes values intact.
-    	*@param s new integer seconds value
-    	*@sa setArcSec()
-    	*/
-    void setHSec( const int &s ) { setH( hour(), minute(), s ); }
-
     /**@short Sets floating-point value of angle, in hours.
-    	*
-    	*Converts argument from hours to degrees, then
-    	*sets floating-point value of angle, in degrees.
-    	*@param x new angle, in hours (double)
-    	*@sa setD()
-    	*/
+     * 
+     * Converts argument from hours to degrees, then
+     * sets floating-point value of angle, in degrees.
+     * @param x new angle, in hours (double)
+     * @sa setD()
+     */
     void setH( const double &x );
 
     /**@short Sets floating-point value of angle, in hours.
-    	*
-    	*Converts argument values from hours to degrees, then
-    	*sets floating-point value of angle, in degrees.
-    	*This is an overloaded member function, provided for convenience.  It
-    	*behaves essentially like the above function.
-    	*@param h integer hours portion of angle
-    	*@param m integer minutes portion of angle
-    	*@param s integer seconds portion of angle
-    	*@param ms integer milliseconds portion of angle
-    	*@sa setD()
-    	*/
+     * 
+     * Converts argument values from hours to degrees, then
+     * sets floating-point value of angle, in degrees.
+     * This is an overloaded member function, provided for convenience.  It
+     * behaves essentially like the above function.
+     * @param h integer hours portion of angle
+     * @param m integer minutes portion of angle
+     * @param s integer seconds portion of angle
+     * @param ms integer milliseconds portion of angle
+     * @sa setD()
+     */
     void setH( const int &h, const int &m, const int &s, const int &ms=0 );
 
-    /**@short Copy value of another dms angle
-    	*@param d set angle according to this dms object
-    	*/
-    void set( const dms &d ) { setD( d.Degrees() ); }
-
-    /**Copy value of another dms angle.  Differs from above function only
-    	*in argument type.  Identical to setD(double d).
-    	*@param d set angle according to this double value
-    	*@sa setD()
-    	*/
-    void set( const double &d ) { setD( d ); }
-
     /**@short Attempt to parse the string argument as a dms value, and set the dms object
      * accordingly.
      * @param s the string to be parsed as a dms value.  The string can be an int or
@@ -290,25 +239,39 @@ public:
     static const double DegToRad;
 
     /**@short Static function to create a DMS object from a QString.
-    	*
-    	*There are several ways to specify the angle:
-    	*@li Integer numbers  ( 5 or -33 )
-    	*@li Floating-point numbers  ( 5.0 or -33.0 )
-    	*@li colon-delimited integers ( 5:0:0 or -33:0:0 )
-    	*@li colon-delimited with float seconds ( 5:0:0.0 or -33:0:0.0 )
-    	*@li colon-delimited with float minutes ( 5:0.0 or -33:0.0 )
-    	*@li space-delimited ( 5 0 0; -33 0 0 ) or ( 5 0.0 or -33 0.0 )
-    	*@li space-delimited, with unit labels ( 5h 0m 0s or -33d 0m 0s )
-    	*@param s the string to be parsed as an angle value
-    	*@param deg if true, s is expressed in degrees; if false, s is expressed in hours
-    	*@return a dms object whose value is parsed from the string argument
-    	*/
+     *
+     * There are several ways to specify the angle:
+     * @li Integer numbers  ( 5 or -33 )
+     * @li Floating-point numbers  ( 5.0 or -33.0 )
+     * @li colon-delimited integers ( 5:0:0 or -33:0:0 )
+     * @li colon-delimited with float seconds ( 5:0:0.0 or -33:0:0.0 )
+     * @li colon-delimited with float minutes ( 5:0.0 or -33:0.0 )
+     * @li space-delimited ( 5 0 0; -33 0 0 ) or ( 5 0.0 or -33 0.0 )
+     * @li space-delimited, with unit labels ( 5h 0m 0s or -33d 0m 0s )
+     * @param s the string to be parsed as an angle value
+     * @param deg if true, s is expressed in degrees; if false, s is expressed in hours
+     * @return a dms object whose value is parsed from the string argument
+     */
     static dms fromString(const QString & s, bool deg);
 
+    dms operator - () { return dms(-D); }
 private:
     double D;
+
+    friend dms operator+(dms, dms);
+    friend dms operator-(dms, dms); 
 };
 
+/// Add two angles
+inline dms operator + (dms a, dms b) {
+    return dms( a.D + b.D);
+}
+
+/// Subtract angles
+inline dms operator - (dms a, dms b) {
+    return dms( a.D - b.D);
+}
+
 // Inline sincos
 inline void dms::SinCos(double& s, double& c) const {
 #ifdef __GLIBC__
@@ -327,4 +290,7 @@ inline void dms::SinCos(double& s, double& c) const {
 #endif
 }
 
+/** Overloaded equality operator */
+inline bool operator == (const dms& a1, const dms& a2) { return a1.Degrees() == a2.Degrees(); }
+
 #endif
