@@ -26,6 +26,9 @@
 #include "skyobjects/ksmoon.h"
 #include "skyobjects/skyobject.h"
 #include "skyobjects/trailobject.h"
+#include "skyobjects/deepskyobject.h"
+#include "skyobjects/ksmoon.h"
+#include "skycomponents/skymapcomposite.h"
 #include "skymap.h"
 
 #include <config-kstars.h>
@@ -38,7 +41,7 @@
 #include "indi/indiproperty.h"
 #endif
 
-#include "skycomponents/constellationboundary.h"
+#include "skycomponents/constellationboundarylines.h"
 
 
 namespace {
@@ -75,7 +78,7 @@ namespace {
                 i18n ("Rise time: %1", time) :
                 i18nc("the time at which an object falls below the horizon", "Set time: %1" , time);
         }
-        if( o->alt()->Degrees() > 0 )
+        if( o->alt().Degrees() > 0 )
             return isRaise ? i18n( "No rise time: Circumpolar" ) : i18n( "No set time: Circumpolar" );
         else
             return isRaise ? i18n( "No rise time: Never rises" ) : i18n( "No set time: Never rises" );
@@ -97,21 +100,21 @@ namespace {
 
 KSPopupMenu::KSPopupMenu()
     : KMenu( KStars::Instance() )
-{
-    ks = KStars::Instance();
-}
+{}
 
 KSPopupMenu::~KSPopupMenu()
 {}
 
-void KSPopupMenu::createEmptyMenu( SkyObject *nullObj ) {
-    initPopupMenu( nullObj, i18n( "Empty sky" ), QString(), QString(), true, true, false, false, false, true, false );
-
+void KSPopupMenu::createEmptyMenu( SkyPoint *nullObj ) {
+    KStars* ks = KStars::Instance();
+    SkyObject o( SkyObject::TYPE_UNKNOWN, nullObj->ra(), nullObj->dec() );
+    initPopupMenu( &o, i18n( "Empty sky" ), QString(), QString(), false, false );
     addAction( i18nc( "Sloan Digital Sky Survey", "Show SDSS Image" ), ks->map(), SLOT( slotSDSS() ) );
-    addAction( i18nc( "Digitized Sky Survey", "Show DSS Image" ), ks->map(), SLOT( slotDSS() ) );
+    addAction( i18nc( "Digitized Sky Survey", "Show DSS Image" ),      ks->map(), SLOT( slotDSS()  ) );
 }
 
 void KSPopupMenu::createStarMenu( StarObject *star ) {
+    KStars* ks = KStars::Instance();
     //Add name, rise/set time, center/track, and detail-window items
 	QString name;
 	if( star->name() != "star" ) {
@@ -133,48 +136,33 @@ void KSPopupMenu::createStarMenu( StarObject *star ) {
         addAction( i18nc( "Digitized Sky Survey", "Show DSS Image" ), ks->map(), SLOT( slotDSS() ) );
     }
 }
- 
-void KSPopupMenu::createDeepSkyObjectMenu( SkyObject *obj ) {
+
+void KSPopupMenu::createDeepSkyObjectMenu( DeepSkyObject *obj ) {
 	QString name = getObjectName(obj);
-    QString typeName = ks->data()->typeName( obj->type() );
+    QString typeName = KStarsData::Instance()->typeName( obj->type() );
 	// FIXME: information about angular sizes should be added.
-	// Requires downcast. Not sure whether it safe.
 	QString info = magToStr( obj->mag() );
-	
 	initPopupMenu( obj, name, typeName, info );
     addLinksToMenu( obj );
 }
 
-void KSPopupMenu::createCustomObjectMenu( SkyObject *obj ) {
-	QString name = getObjectName(obj); 
-    QString typeName = ks->data()->typeName( obj->type() );
-	QString info = magToStr( obj->mag() );
-	
-    initPopupMenu( obj, name, typeName, info );
-
-    addLinksToMenu( obj, true );
-}
-
 void KSPopupMenu::createPlanetMenu( SkyObject *p ) {
-    bool addTrail( ! ((TrailObject*)p)->hasTrail() );
-    QString info;
-	QString type;
-    if ( p->name() == "Moon" ) {
-        info = QString("%1, %2").arg( magToStr(p->mag()), reinterpret_cast<KSMoon*>(p)->phaseName() );
-    } else {
-		// FIXME: angular size is required.
-		info = magToStr( p->mag() );
-		type = i18n("Solar system object");
-	}
-    initPopupMenu( p, p->translatedName(), type, info, true, true, true, true, addTrail );
+    QString info = magToStr( p->mag() );
+	QString type = i18n("Solar system object");;
+    initPopupMenu( p, p->translatedName(), type, info);
     addLinksToMenu( p, false ); //don't offer DSS images for planets
 }
 
+void KSPopupMenu::createMoonMenu( KSMoon *moon ) {
+    QString info = QString("%1, %2").arg( magToStr(moon->mag()), moon->phaseName() );
+    initPopupMenu( moon, moon->translatedName(), QString(), info);
+    addLinksToMenu( moon, false ); //don't offer DSS images for planets
+}
+
 void KSPopupMenu::initPopupMenu( SkyObject *obj, QString name, QString type, QString info,
-                                 bool showRiseSet, bool showCenterTrack, bool showDetails, bool showTrail, bool addTrail,
-                                 bool showAngularDistance, bool showObsList )
+                                 bool showDetails, bool showObsList )
 {
-    ks = KStars::Instance();
+    KStars* ks = KStars::Instance();
 
     clear();
     bool showLabel = name != i18n("star") && !name.isEmpty();
@@ -182,73 +170,53 @@ void KSPopupMenu::initPopupMenu( SkyObject *obj, QString name, QString type, QSt
         name = i18n( "Empty sky" );
 
     addFancyLabel( name );
-    if ( !type.isEmpty() )
-        addFancyLabel( type );
-    if ( !info.isEmpty() )
-        addFancyLabel( info );
-    addFancyLabel( ConstellationBoundary::Instance()->constellationName( obj ) );
+    addFancyLabel( type );
+    addFancyLabel( info );
+    addFancyLabel( KStarsData::Instance()->skyComposite()->getConstellationBoundary()->constellationName( obj ) );
 
     //Insert Rise/Set/Transit labels
-    if( showRiseSet && obj ) {
-        SkyObject* o = obj->clone();
-        addSeparator();
-        addFancyLabel( riseSetTimeLabel(o, true),  -2 );
-        addFancyLabel( riseSetTimeLabel(o, false), -2 );
-        addFancyLabel( transitTimeLabel(o),        -2 );
-        addSeparator();
-        delete o;
-    }
-
+    SkyObject* o = obj->clone();
+    addSeparator();
+    addFancyLabel( riseSetTimeLabel(o, true),  -2 );
+    addFancyLabel( riseSetTimeLabel(o, false), -2 );
+    addFancyLabel( transitTimeLabel(o),        -2 );
+    addSeparator();
+    delete o;
     //Insert item for centering on object
-    if ( showCenterTrack && obj ) {
-        addAction( i18n( "Center && Track" ), ks->map(), SLOT( slotCenter() ) );
-    }
-
+    addAction( i18n( "Center && Track" ), ks->map(), SLOT( slotCenter() ) );
     //Insert item for measuring distances
     //FIXME: add key shortcut to menu items properly!
-    if ( showAngularDistance && obj ) {
-        if (! (ks->map()->isAngleMode()) ) {
-            addAction( i18n( "Angular Distance To...            [" ), ks->map(),
-                       SLOT( slotBeginAngularDistance() ) );
-        } else {
-            addAction( i18n( "Compute Angular Distance          ]" ), ks->map(),
-                       SLOT( slotEndAngularDistance() ) );
-        }
-    }
-
+    addAction( i18n( "Angular Distance To...            [" ), ks->map(),
+               SLOT( slotBeginAngularDistance() ) );
 
     //Insert item for Showing details dialog
-    if ( showDetails && obj ) {
-        addAction( i18nc( "Show Detailed Information Dialog", "Details" ), ks->map(),
-                   SLOT( slotDetail() ) );
-    }
-
+    if ( showDetails )
+        addAction( i18nc( "Show Detailed Information Dialog", "Details" ), ks->map(), SLOT( slotDetail() ) );
     //Insert "Add/Remove Label" item
-    if ( showLabel && obj ) {
+    if ( showLabel ) {
         if ( ks->map()->isObjectLabeled( obj ) ) {
             addAction( i18n( "Remove Label" ), ks->map(), SLOT( slotRemoveObjectLabel() ) );
         } else {
             addAction( i18n( "Attach Label" ), ks->map(), SLOT( slotAddObjectLabel() ) );
         }
     }
-
-    if ( showObsList && obj ) {
+    // Should show observing list
+    if( showObsList ) {
         if ( ks->observingList()->contains( obj ) )
             addAction( i18n("Remove From Observing WishList"), ks->observingList(), SLOT( slotRemoveObject() ) );
         else
             addAction( i18n("Add to Observing WishList"), ks->observingList(), SLOT( slotAddObject() ) );
     }
-
-    if ( showTrail && obj && obj->isSolarSystem() ) {
-        if ( addTrail ) {
-            addAction( i18n( "Add Trail" ), ks->map(), SLOT( slotAddPlanetTrail() ) );
-        } else {
+    // Should we show trail actions
+    TrailObject* t = dynamic_cast<TrailObject*>( obj );
+    if( t ) {
+        if( t->hasTrail() )
             addAction( i18n( "Remove Trail" ), ks->map(), SLOT( slotRemovePlanetTrail() ) );
-        }
+        else
+            addAction( i18n( "Add Trail" ), ks->map(), SLOT( slotAddPlanetTrail() ) );
     }
 
     addSeparator();
-
 #ifdef HAVE_XPLANET
     if ( obj->isSolarSystem() && obj->type() != SkyObject::COMET ) {
         QMenu *xplanetSubmenu = new QMenu();
@@ -258,16 +226,12 @@ void KSPopupMenu::initPopupMenu( SkyObject *obj, QString name, QString type, QSt
         addMenu( xplanetSubmenu );
     }
 #endif
-
     addSeparator();
-
-    #ifdef HAVE_INDI_H
-    if (addINDI())
-        addSeparator();
-    #endif
+    addINDI();
 }
 
 void KSPopupMenu::addLinksToMenu( SkyObject *obj, bool showDSS ) {
+    KStars* ks = KStars::Instance();
     QString sURL;
     QStringList::ConstIterator itList, itTitle, itListEnd;
 
@@ -275,7 +239,6 @@ void KSPopupMenu::addLinksToMenu( SkyObject *obj, bool showDSS ) {
     itTitle = obj->ImageTitle().constBegin();
     itListEnd = obj->ImageList().constEnd();
 
-    int id = 100;
     for ( ; itList != itListEnd; ++itList ) {
         QString t = QString(*itTitle);
         sURL = QString(*itList);
@@ -286,15 +249,15 @@ void KSPopupMenu::addLinksToMenu( SkyObject *obj, bool showDSS ) {
     if ( showDSS ) {
         addAction( i18nc( "Sloan Digital Sky Survey", "Show SDSS Image" ), ks->map(), SLOT( slotSDSS() ) );
         addAction( i18nc( "Digitized Sky Survey", "Show DSS Image" ), ks->map(), SLOT( slotDSS() ) );
-        addSeparator();
     }
-    else if ( obj->ImageList().count() ) addSeparator();
+
+    if( obj->ImageList().count() || showDSS )
+        addSeparator();
 
     itList  = obj->InfoList().constBegin();
     itTitle = obj->InfoTitle().constBegin();
     itListEnd = obj->InfoList().constEnd();
 
-    id = 200;
     for ( ; itList != itListEnd; ++itList ) {
         QString t = QString(*itTitle);
         sURL = QString(*itList);
@@ -303,10 +266,10 @@ void KSPopupMenu::addLinksToMenu( SkyObject *obj, bool showDSS ) {
     }
 }
 
-bool KSPopupMenu::addINDI(void)
+void KSPopupMenu::addINDI()
 {
-    #ifdef HAVE_INDI_H
-    INDIMenu *indiMenu = ks->indiMenu();
+#ifdef HAVE_INDI_H
+    INDIMenu *indiMenu = KStars::Instance()->indiMenu();
     DeviceManager *managers;
     INDI_D *dev;
     INDI_G *grp;
@@ -314,7 +277,7 @@ bool KSPopupMenu::addINDI(void)
     INDI_E *element;
 
     if (indiMenu->managers.count() == 0)
-        return false;
+        return;
 
     foreach ( managers, indiMenu->managers )
     {
@@ -323,7 +286,7 @@ bool KSPopupMenu::addINDI(void)
             if (!dev->INDIStdSupport)
                 continue;
 
-            menuDevice = new KMenu(dev->label);
+            KMenu* menuDevice = new KMenu(dev->label);
             addMenu(menuDevice);
 
             foreach (grp, dev->gl )
@@ -362,10 +325,7 @@ bool KSPopupMenu::addINDI(void)
                         if (prop->stdID == ON_COORD_SET)
                             continue;
 
-                        if (element->state == PS_ON)
-                            a->setChecked(true);
-                        else
-                            a->setChecked(false);
+                        a->setChecked( element->state == PS_ON );
                     }
                 } // end property
             } // end group
@@ -380,15 +340,14 @@ bool KSPopupMenu::addINDI(void)
             }
         } // end device
     } // end device manager
-
-    return true;
-    #else
-    return false;
-    #endif
+    addSeparator();
+#endif
 }
 
 
 void KSPopupMenu::addFancyLabel(QString name, int deltaFontSize) {
+    if( name.isEmpty() )
+        return;
     QLabel* label = new QLabel( "<b>"+name+"</b>", this );
     label->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
     if( deltaFontSize != 0 ) {
