@@ -20,7 +20,6 @@
 
 #include <QFile>
 #include <QPen>
-#include <QPainter>
 #include <kglobal.h>
 
 #include "Options.h"
@@ -30,17 +29,42 @@
 #include "ksfilereader.h"
 #include "skymap.h"
 #include "skylabeler.h"
+#include "skypainter.h"
+#include "projections/projector.h"
 
-CometsComponent::CometsComponent( SolarSystemComposite *parent, bool (*visibleMethod)(), int msize )
-        : SolarSystemListComponent( parent, visibleMethod, msize )
+CometsComponent::CometsComponent( SolarSystemComposite *parent )
+        : SolarSystemListComponent( parent )
 {
+    loadData();
 }
 
-CometsComponent::~CometsComponent() {
-    //object deletion handled in grandparent class (ListComponent)
+CometsComponent::~CometsComponent()
+{}
+
+bool CometsComponent::selected() {
+    return Options::showComets();
 }
 
-void CometsComponent::init() {
+/*
+ * @short Initialize the asteroids list.
+ * Reads in the asteroids data from the asteroids.dat file.
+ *
+ * Populate the list of Comets from the data file.
+ * Each line in the data file is parsed as follows:
+ * @li 3-37 Name [string]
+ * @li 38-42 Modified Julian Day of orbital elements [int]
+ * @li 44-53 semi-major axis of orbit in AU [double]
+ * @li 55-64 eccentricity of orbit [double]
+ * @li 66-74 inclination angle of orbit in degrees [double]
+ * @li 76-84 argument of perihelion in degrees [double]
+ * @li 86-94 Longitude of the Ascending Node in degrees [double]
+ * @li 82-93 Date of most proximate perihelion passage (YYYYMMDD.DDD) [double]
+ * @li 124-127 Absolute magnitude [float]
+ * @li 129-132 Slope parameter [float]
+ *
+ * @note See KSComet constructor for more details.
+ */
+void CometsComponent::loadData() {
     QFile file;
 
     if ( KSUtils::openDataFile( file, "comets.dat" ) ) {
@@ -79,7 +103,7 @@ void CometsComponent::init() {
             com = new KSComet( name, QString(), JD, q, e, dms(dble_i), dms(dble_w), dms(dble_N), Tp, H, G );
             com->setAngularSize( 0.005 );
 
-            objectList().append( com );
+            m_ObjectList.append( com );
 
             //Add *short* name to the list of object names
             objectNames(SkyObject::COMET).append( com->name() );
@@ -87,41 +111,22 @@ void CometsComponent::init() {
     }
 }
 
-void CometsComponent::draw( QPainter& psky )
+void CometsComponent::draw( SkyPainter *skyp )
 {
-    if ( !visible() || Options::zoomFactor() < 10*MINZOOM ) return;
+    if( !selected() || Options::zoomFactor() < 10*MINZOOM )
+        return;
 
-    SkyMap *map = SkyMap::Instance();
-
-    bool hideLabels =  ! Options::showCometNames() || (map->isSlewing() && Options::hideLabels() );
+    bool hideLabels =  ! Options::showCometNames() || (SkyMap::Instance()->isSlewing() && Options::hideLabels() );
     double rsunLabelLimit = Options::maxRadCometName();
 
-    psky.setPen( QPen( QColor( "darkcyan" ) ) );
-    psky.setBrush( QBrush( QColor( "darkcyan" ) ) );
+    //FIXME: Should these be config'able?
+    skyp->setPen( QPen( QColor( "darkcyan" ) ) );
+    skyp->setBrush( QBrush( QColor( "darkcyan" ) ) );
 
-    foreach ( SkyObject *so, objectList() ) {
+    foreach ( SkyObject *so, m_ObjectList ) {
         KSComet *com = (KSComet*)so;
-
-        if ( ! map->checkVisibility( com ) ) continue;
-
-        QPointF o = map->toScreen( com );
-
-        if ( ! map->onScreen( o ) ) continue;
-        //if ( ( o.x() >= 0. && o.x() <= Width && o.y() >= 0. && o.y() <= Height ) )
-
-        float size = com->angSize() * map->scale() * dms::PI * Options::zoomFactor()/10800.0;
-	if ( size < 1.0 ) {
-            psky.drawPoint( o );
-        } else {
-            float x1 = o.x() - 0.5*size;
-            float y1 = o.y() - 0.5*size;
-
-            psky.drawEllipse( QRectF( x1, y1, size, size ) );
-        }
-
-//        float tailsize = com->getTailAngSize().Degrees() * map->scale() * dms::PI * Options::zoomFactor()/10800.0;
-
-        if ( hideLabels || com->rsun() >= rsunLabelLimit ) continue;
-        SkyLabeler::AddLabel( o, com, SkyLabeler::COMET_LABEL );
+        bool drawn = skyp->drawPointSource(com,com->mag());
+        if ( drawn && !(hideLabels || com->rsun() >= rsunLabelLimit) )
+            SkyLabeler::AddLabel( com, SkyLabeler::COMET_LABEL );
     }
 }
