@@ -39,7 +39,7 @@ public:
      *@short Constructor
      *@p parent Pointer to the parent SkyComposite object
      */
-    HorizonComponent(SkyComponent *parent );
+    HorizonComponent(SkyComposite *parent );
 
     /**
      *@short Destructor
@@ -51,19 +51,14 @@ public:
      *@p map Pointer to the SkyMap object
      *@p psky Reference to the QPainter on which to paint
      */
-    virtual void draw( QPainter& psky );
-
-    /**
-     *@short Initialize the Horizon
-     */
-    virtual void init();
+    virtual void draw( SkyPainter *skyp );
 
     virtual void update( KSNumbers* );
 
     bool selected();
 
 private:
-    void drawCompassLabels( QPainter& psky );
+    void drawCompassLabels( SkyPainter *skyp );
 };
 
 #endif
