@@ -17,18 +17,14 @@
 
 #include "starcomponent.h"
 
-#include <QPixmap>
-#include <QPainter>
-
-#include <QRectF>
-#include <QFontMetricsF>
 #include <kglobal.h>
 
 #include "Options.h"
 #include "kstarsdata.h"
-#include "ksutils.h"
 #include "skymap.h"
 #include "skyobjects/starobject.h"
+#include "skyqpainter.h"
+#include "skypainter.h"
 
 #include "skymesh.h"
 #include "skylabel.h"
@@ -38,19 +34,22 @@
 #include "binfilehelper.h"
 #include "starblockfactory.h"
 
+#include "projections/projector.h"
+
+
 #if defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
 #include <sys/endian.h>
 #define bswap_16(x) bswap16(x)
 #define bswap_32(x) bswap32(x)
 #else
-#include <byteswap.h>
+#include "byteorder.h"
 #endif
 
 #include <kde_file.h>
 
 StarComponent *StarComponent::pinstance = 0;
 
-StarComponent::StarComponent(SkyComponent *parent )
+StarComponent::StarComponent(SkyComposite *parent )
     : ListComponent(parent), m_reindexNum(J2000), m_FaintMagnitude(-5.0), 
       starsLoaded(false), focusStar(NULL)
 {
@@ -65,18 +64,23 @@ StarComponent::StarComponent(SkyComponent *parent )
     m_reindexInterval = StarObject::reindexInterval( 304.0 );
 
     m_zoomMagLimit = 0.0;
-    m_reloadSplash = m_reindexSplash = 0;
-    m_validLineNums = false;
 
     for ( int i = 0; i <= MAX_LINENUMBER_MAG; i++ )
         m_labelList[ i ] = new LabelList;
+
+    // Actually load data
+    emitProgressText( i18n("Loading stars" ) );
+    loadStaticData();
+    // Load any deep star catalogs that are available
+    loadDeepStarCatalogs();
+    SkyQPainter::initStarImages();
 }
 
 StarComponent::~StarComponent() {
     // Empty
 }
 
-StarComponent *StarComponent::Create( SkyComponent *parent ) {
+StarComponent *StarComponent::Create( SkyComposite *parent ) {
     if( pinstance )
         delete pinstance;
     pinstance = new StarComponent( parent );
@@ -87,27 +91,12 @@ bool StarComponent::selected() {
     return Options::showStars();
 }
 
-void StarComponent::init(KStarsData *data) {
-    emitProgressText( i18n("Loading stars" ) );
-    m_Data = data;
-
-    loadStaticData();
-
-    // Load any deep star catalogs that are available
-    loadDeepStarCatalogs();
-
-    StarObject::initImages();
-}
-
 bool StarComponent::addDeepStarCatalogIfExists( const QString &fileName, float trigMag, bool staticstars ) {
     if( BinFileHelper::testFileExists( fileName ) ) {
-        DeepStarComponent *newdsc;
-        m_DeepStarComponents.append( newdsc = new DeepStarComponent( this, fileName, trigMag, staticstars ) );
-        newdsc->init( KStarsData::Instance() );
+        m_DeepStarComponents.append( new DeepStarComponent( parent(), fileName, trigMag, staticstars ) );
         return true;
     }
-    else
-        return false;
+    return false;
 }
 
 
@@ -130,11 +119,8 @@ int StarComponent::loadDeepStarCatalogs() {
 
 //This function is empty for a reason; we override the normal 
 //update function in favor of JiT updates for stars.
-void StarComponent::update( KStarsData *data, KSNumbers *num )   
-{   
-    Q_UNUSED(data)   
-    Q_UNUSED(num)   
-}   
+void StarComponent::update( KSNumbers*)
+{}
 
 // We use the update hook to re-index all the stars when the date has changed by
 // more than 150 years.
@@ -181,9 +167,9 @@ void StarComponent::reindexAll( KSNumbers *num )
     }
 
     // re-populate it from the objectList
-    int size = objectList().size();
+    int size = m_ObjectList.size();
     for ( int i = 0; i < size; i++ ) {
-        StarObject* star = (StarObject*) objectList()[ i ];
+        StarObject* star = (StarObject*) m_ObjectList[ i ];
         Trixel trixel = m_skyMesh->indexStar( star );
         m_starIndex->at( trixel )->append( star );
     }
@@ -208,43 +194,11 @@ float StarComponent::faintMagnitude() const {
     return faintmag;
 }
 
-float StarComponent::starRenderingSize( float mag ) const {
-    //adjust maglimit for ZoomLevel
-    const double maxSize = 10.0;
-
-    double lgmin = log10(MINZOOM);
-    double lgmax = log10(MAXZOOM);
-    double lgz = log10(Options::zoomFactor());
-
-    // Old formula:
-    //    float sizeMagLim = ( 2.000 + 2.444 * Options::memUsage() / 10.0 ) * ( lgz - lgmin ) + 5.8;
-
-    // Using the maglim to compute the sizes of stars reduces
-    // discernability between brighter and fainter stars at high zoom
-    // levels. To fix that, we use an "arbitrary" constant in place of
-    // the variable star density.
-    // Not using this formula now.
-    //    float sizeMagLim = 4.444 * ( lgz - lgmin ) + 5.0;
-
-    float sizeMagLim = zoomMagnitudeLimit();
-    if( sizeMagLim > faintMagnitude() * ( 1 - 1.5/16 ) )
-        sizeMagLim = faintMagnitude() * ( 1 - 1.5/16 );
-
-    float sizeFactor = maxSize + (lgz - lgmin);
-
-    float size = ( sizeFactor*( sizeMagLim - mag ) / sizeMagLim ) + 1.;
-    if( size <= 1.0 ) size = 1.0;
-    if( size > maxSize ) size = maxSize;
-
-    return size;
-}
-
-float StarComponent::zoomMagnitudeLimit() const {
+float StarComponent::zoomMagnitudeLimit() {
 
     //adjust maglimit for ZoomLevel
     double lgmin = log10(MINZOOM);
-    double lgmax = log10(MAXZOOM);
-    double lgz = log10(Options::zoomFactor());
+    double lgz   = log10(Options::zoomFactor());
 
     // Old formula:
     //    float maglim = ( 2.000 + 2.444 * Options::memUsage() / 10.0 ) * ( lgz - lgmin ) + Options::magLimitDrawStarZoomOut();
@@ -270,55 +224,54 @@ float StarComponent::zoomMagnitudeLimit() const {
     // Reducing the slope w.r.t zoom factor to avoid the extremely fast increase in star density with zoom
     // that 4.444 gives us (although that is what the derivation gives us)
 
-    float maglim = 3.7 * ( lgz - lgmin ) + 2.222 * log10( Options::starDensity() ) + 3.5;
-
-    return maglim;
-
+    return 3.5 + 3.7*( lgz - lgmin ) + 2.222*log10( static_cast<float>(Options::starDensity()) );
 }
 
-void StarComponent::draw( QPainter& psky )
+void StarComponent::draw( SkyPainter *skyp )
 {
-    if ( ! selected() ) return;
+    if( !selected() )
+        return;
 
-    SkyMap *map = SkyMap::Instance();
-    KStarsData* data = KStarsData::Instance();
-    UpdateID updateID = data->updateID();
+    SkyMap *map             = SkyMap::Instance();
+    const Projector *proj   = map->projector();
+    KStarsData* data        = KStarsData::Instance();
+    UpdateID updateID       = data->updateID();
 
     bool checkSlewing = ( map->isSlewing() && Options::hideOnSlew() );
-    m_hideLabels =  ( map->isSlewing() && Options::hideLabels() ) ||
-                    ! ( Options::showStarMagnitudes() || Options::showStarNames() );
+    m_hideLabels = checkSlewing || !( Options::showStarMagnitudes() || Options::showStarNames() );
 
     //shortcuts to inform whether to draw different objects
-    bool hideFaintStars( checkSlewing && Options::hideStars() );
+    bool hideFaintStars = checkSlewing && Options::hideStars();
     double hideStarsMag = Options::magLimitHideStar();
     reindex( data->updateNum() );
 
     double lgmin = log10(MINZOOM);
     double lgmax = log10(MAXZOOM);
-    double lgz = log10(Options::zoomFactor());
+    double lgz   = log10(Options::zoomFactor());
 
     double maglim;
     m_zoomMagLimit = maglim = zoomMagnitudeLimit();
 
     double labelMagLim = Options::starLabelDensity() / 5.0;
     labelMagLim += ( 12.0 - labelMagLim ) * ( lgz - lgmin) / (lgmax - lgmin );
-    if ( labelMagLim > 8.0 ) labelMagLim = 8.0;
+    if( labelMagLim > 8.0 )
+        labelMagLim = 8.0;
 
-//REMOVE
-//     //Set the brush
-//     QColor fillColor( Qt::white );
-//     if ( starColorMode() == 1 ) fillColor = Qt::red;
-//     if ( starColorMode() == 2 ) fillColor = Qt::black;
-//     psky.setBrush( QBrush( fillColor ) );
-//     if ( starColorMode() > 0 )
-//         psky.setPen( QPen( fillColor ) );
-//     else
-//        //Reset the colors before drawing the stars.
-//        //Strictly speaking, we don't need to do this every time, but once per
-//        //draw loop isn't too expensive.
-//        StarObject::updateColors( (! Options::useAntialias() ||
-//                                   map->isSlewing()), starColorIntensity() );
-//END_REMOVE
+    //Calculate sizeMagLim
+    // Old formula:
+    //    float sizeMagLim = ( 2.000 + 2.444 * Options::memUsage() / 10.0 ) * ( lgz - lgmin ) + 5.8;
+
+    // Using the maglim to compute the sizes of stars reduces
+    // discernability between brighter and fainter stars at high zoom
+    // levels. To fix that, we use an "arbitrary" constant in place of
+    // the variable star density.
+    // Not using this formula now.
+    //    float sizeMagLim = 4.444 * ( lgz - lgmin ) + 5.0;
+
+    float sizeMagLim = zoomMagnitudeLimit();
+    if( sizeMagLim > faintMagnitude() * ( 1 - 1.5/16 ) )
+        sizeMagLim = faintMagnitude() * ( 1 - 1.5/16 );
+    skyp->setSizeMagLimit(sizeMagLim);
 
     //Loop for drawing star images
 
@@ -329,16 +282,14 @@ void StarComponent::draw( QPainter& psky )
 
     int nTrixels = 0;
 
-    visibleStarCount = 0;
-
-    while ( region.hasNext() ) {
+    while( region.hasNext() ) {
         ++nTrixels;
         Trixel currentRegion = region.next();
         StarList* starList = m_starIndex->at( currentRegion );
         for (int i=0; i < starList->size(); ++i) {
-        
             StarObject *curStar = starList->at( i );
-            if( !curStar ) continue;
+            if( !curStar )
+                continue;
             
             if ( curStar->updateID != updateID )
                 curStar->JITupdate( data );
@@ -346,22 +297,14 @@ void StarComponent::draw( QPainter& psky )
             float mag = curStar->mag();
             
             // break loop if maglim is reached
-            if ( mag > maglim || ( hideFaintStars && curStar->mag() > hideStarsMag ) ) {
+            if ( mag > maglim || ( hideFaintStars && curStar->mag() > hideStarsMag ) )
                 break;
-            }
                  
-            if ( ! map->checkVisibility( curStar ) ) continue;
-            QPointF o = map->toScreen( curStar );
-            
-            if ( ! map->onScreen( o ) ) continue;
+            bool drawn = skyp->drawPointSource( curStar, mag, curStar->spchar() );
 
-            curStar->draw( psky, o.x(), o.y(), starRenderingSize( mag ), (starColorMode()==0),
-                           starColorIntensity(), true );
-            visibleStarCount++;
-            
-            if ( m_hideLabels || mag > labelMagLim ) continue;
-            
-            addLabel( o, curStar );
+            //FIXME_SKYPAINTER: find a better way to do this.
+            if ( drawn && !(m_hideLabels || mag > labelMagLim) )
+                addLabel( proj->toScreen(curStar), curStar );
         }
     }
 
@@ -370,19 +313,12 @@ void StarComponent::draw( QPainter& psky )
         if ( focusStar->updateID != updateID )
             focusStar->JITupdate( data );
         float mag = focusStar->mag();
-        if ( map->checkVisibility( focusStar ) ) {
-            QPointF o = map->toScreen( focusStar );
-            if ( map->onScreen( o ) ) {
-                focusStar->draw( psky, o.x(), o.y(), starRenderingSize( mag ), (starColorMode()==0),
-                                 starColorIntensity(), true );
-                visibleStarCount++;
-            }
-        }
+        skyp->drawPointSource(focusStar, mag, focusStar->spchar() );
     }
 
     // Now draw each of our DeepStarComponents
     for( int i =0; i < m_DeepStarComponents.size(); ++i ) {
-        m_DeepStarComponents.at( i )->draw( psky );
+        m_DeepStarComponents.at( i )->draw( skyp );
     }
 }
 
@@ -394,11 +330,13 @@ void StarComponent::addLabel( const QPointF& p, StarObject *star )
     m_labelList[ idx ]->append( SkyLabel( p, star ) );
 }
 
-void StarComponent::drawLabels( QPainter& psky )
+void StarComponent::drawLabels()
 {
-    if ( m_hideLabels ) return;
+    if( m_hideLabels )
+        return;
 
-    psky.setPen( QColor( KStarsData::Instance()->colorScheme()->colorNamed( "SNameColor" ) ) );
+    SkyLabeler *labeler = SkyLabeler::Instance();
+    labeler->setPen( QColor( KStarsData::Instance()->colorScheme()->colorNamed( "SNameColor" ) ) );
 
     int max = int( m_zoomMagLimit * 10.0 );
     if ( max < 0 ) max = 0;
@@ -407,7 +345,7 @@ void StarComponent::drawLabels( QPainter& psky )
     for ( int i = 0; i <= max; i++ ) {
         LabelList* list = m_labelList[ i ];
         for ( int j = 0; j < list->size(); j++ ) {
-            list->at(j).obj->drawNameLabel( psky, list->at(j).o );
+            labeler->drawNameLabel( list->at(j).obj, list->at(j).o );
         }
         list->clear();
     }
@@ -418,6 +356,7 @@ bool StarComponent::loadStaticData()
 {
     // We break from Qt / KDE API and use traditional file handling here, to obtain speed.
     // We also avoid C++ constructors for the same reason.
+    KStarsData* data = KStarsData::Instance();
     FILE *dataFile, *nameFile;
     bool swapBytes = false;
     BinFileHelper dataReader, nameReader;
@@ -515,7 +454,7 @@ bool StarComponent::loadStaticData()
             star = new StarObject;
             star->init( &stardata );
             star->setNames( name, visibleName );
-            star->EquatorialToHorizontal( data()->lst(), data()->geo()->lat() );
+            star->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
             ++nstars;
             
             if ( ! gname.isEmpty() ) m_genName.insert( gname, star );
@@ -527,7 +466,7 @@ bool StarComponent::loadStaticData()
                 objectNames(SkyObject::STAR).append( star -> gname(false) );
             }
                 
-            objectList().append( star );
+            m_ObjectList.append( star );
                 
             m_starIndex->at( trixel )->append( star );
             double pm = star->pmMagnitude();
@@ -556,18 +495,30 @@ SkyObject* StarComponent::findStarByGenetiveName( const QString name ) {
 
 // Overrides ListComponent::findByName() to include genetive name and HD index also in the search
 SkyObject* StarComponent::findByName( const QString &name ) {
-    SkyObject *o;
-    foreach ( o, objectList() )
+    foreach(SkyObject* o, m_ObjectList) {
         if ( QString::compare( o->name(), name, Qt::CaseInsensitive ) == 0 || 
              QString::compare( o->longname(), name, Qt::CaseInsensitive ) == 0 || 
              QString::compare( o->name2(), name, Qt::CaseInsensitive ) == 0 || 
              QString::compare( ((StarObject *)o)->gname(false), name, Qt::CaseInsensitive ) == 0)
             return o;
-
+    }
     return 0;
 }
 
+void StarComponent::objectsInArea( QList<SkyObject*>& list, const SkyRegion& region ) 
+{
+    for( SkyRegion::const_iterator it = region.constBegin(); it != region.constEnd(); it++ )
+    {
+        Trixel trixel = it.key();
+        StarList* starlist = m_starIndex->at( trixel );
+        for( int i = 0; starlist && i < starlist->size(); i++ )
+            if( starlist->at(i) && starlist->at(i)->name() != QString("star") )
+                list.push_back( starlist->at(i) );
+    }
+}
+
 SkyObject *StarComponent::findByHDIndex( int HDnum ) {
+    KStarsData* data = KStarsData::Instance();
     SkyObject *o;
     BinFileHelper hdidxReader;
     // First check the hash to see if we have a corresponding StarObject already
@@ -596,8 +547,8 @@ SkyObject *StarComponent::findByHDIndex( int HDnum ) {
             byteSwap( &stardata );
         }
         m_starObject.init( &stardata );
-        m_starObject.EquatorialToHorizontal( data()->lst(), data()->geo()->lat() );
-        m_starObject.JITupdate( data() );
+        m_starObject.EquatorialToHorizontal( data->lst(), data->geo()->lat() );
+        m_starObject.JITupdate( data );
         focusStar = &m_starObject;
         hdidxReader.closeFile();
         return focusStar;
@@ -637,26 +588,45 @@ SkyObject* StarComponent::objectNearest( SkyPoint *p, double &maxrad )
     double rTry, rBest;
     SkyObject *oTry;
     rBest = maxrad;
-    rTry = maxrad;
+    rTry  = maxrad;
     for( int i = 0; i < m_DeepStarComponents.size(); ++i ) {
-      oTry = m_DeepStarComponents.at( i )->objectNearest( p, rTry );
-      // TODO: Should we multiply rBest by a factor < 1, so that we give higher priority to named stars?
-      if( rTry < rBest ) {
-	rBest = rTry;
-	oBest = oTry;
-      }
+        oTry = m_DeepStarComponents.at( i )->objectNearest( p, rTry );
+        // TODO: Should we multiply rBest by a factor < 1, so that we give higher priority to named stars?
+        if( rTry < rBest ) {
+            rBest = rTry;
+            oBest = oTry;
+        }
     }
     maxrad = rBest;
 
-    return (SkyObject*) oBest;
+    return oBest;
 }
 
-int StarComponent::starColorMode( void ) const {
-    return m_Data->colorScheme()->starColorMode();
-}
+void StarComponent::starsInAperture( QList<StarObject*> &list, const SkyPoint &center, float radius, float maglim )
+{
+    m_skyMesh->intersect( center.ra0().Degrees(), center.dec0().Degrees(), radius, (BufNum) OBJ_NEAREST_BUF );
 
-int StarComponent::starColorIntensity( void ) const {
-    return m_Data->colorScheme()->starColorIntensity();
+    MeshIterator region( m_skyMesh, OBJ_NEAREST_BUF );
+
+    if( maglim < -28 )
+        maglim = m_FaintMagnitude;
+    
+    while ( region.hasNext() ) {
+        Trixel currentRegion = region.next();
+        StarList* starList = m_starIndex->at( currentRegion );
+        for (int i=0; i < starList->size(); ++i) {
+            StarObject* star =  starList->at( i );
+            if( !star ) continue;
+            if ( star->mag() > m_FaintMagnitude ) continue;
+            if( star->angularDistanceTo( &center ).Degrees() <= radius )
+                list.append( star );
+        }
+    }
+    
+    // Add stars from the DeepStarComponents as well
+    for( int i =0; i < m_DeepStarComponents.size(); ++i ) {
+        m_DeepStarComponents.at( i )->starsInAperture( list, center, radius, maglim );
+    }
 }
 
 void StarComponent::byteSwap( starData *stardata ) {
@@ -700,7 +670,6 @@ void StarComponent::printDebugInfo() {
     printf( "Number of visible StarBlocks                 = %8d\n", nBlocks );
     printf( "Number of StarBlocks allocated via SBF       = %8d\n", m_StarBlockFactory.getBlockCount() );
     printf( "Number of unnamed stars in memory            = %8ld\n", nStars );
-    printf( "Number of visible stars (named + unnamed)    = %8ld\n", visibleStarCount );
     printf( "Magnitude of the faintest star in memory     = %8.2f\n", faintMag );
     printf( "Target magnitude limit                       = %8.2f\n", magLim );
     printf( "Size of each StarBlock                       = %8d bytes\n", sizeof( StarBlock ) );
@@ -713,7 +682,6 @@ void StarComponent::printDebugInfo() {
     printf( "Time taken for dynamic load of data          = %8ld ms\n", t_dynamicLoad );
     printf( "Time taken for updating LRU cache            = %8ld ms\n", t_updateCache );
     printf( "Time taken for drawing unnamed stars         = %8ld ms\n", t_drawUnnamed );
-    printf( "Average Number of stars draw per millisecond = %8.2f stars/ms\n", ( (double)visibleStarCount ) / ( t_drawUnnamed + t_drawNamed ) );
     printf( "================================================================\n" );
 }
 
