@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -36,8 +36,6 @@ namespace Marble
 MainWindow::MainWindow( const QString& marbleDataPath, QWidget *parent )
     : KXmlGuiWindow( parent )
 {
-    MarbleGlobal::getInstance()->setProfiles( MarbleGlobal::detectProfiles() );
-    
     m_part = new MarblePart( this, this, QStringList() << marbleDataPath );
 
     setCentralWidget( m_part->widget() );
@@ -53,6 +51,7 @@ MainWindow::MainWindow( const QString& marbleDataPath, QWidget *parent )
     // Creating the plugin menus
     m_part->createInfoBoxesMenu();
     m_part->createOnlineServicesMenu();
+	m_part->createFolderList();
 
     setAutoSaveSettings();
 
