@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -22,6 +22,7 @@
 // Qt
 #include <QtCore/QAbstractItemModel>
 #include <QtCore/QTime>
+#include <QtCore/QTimer>
 #include <QtGui/QItemSelectionModel>
 #include <QtGui/QSizePolicy>
 #include <QtGui/QRegion>
@@ -31,17 +32,20 @@
 #endif
 
 // Marble
+#include "AbstractFloatItem.h"
 #include "AbstractProjection.h"
 #include "AbstractScanlineTextureMapper.h"
-#include "FileStorageWatcher.h"
-#include "FileViewModel.h"
+#include "GeoDataDocument.h"
 #include "GeoDataFeature.h"
 #include "GeoDataLatLonAltBox.h"
+#include "GeoDataTreeModel.h"
 #include "GeoPainter.h"
 #include "GeoSceneDocument.h"
 #include "GeoSceneHead.h"
+#include "GeoSceneMap.h"
+#include "GeoSceneVector.h"
 #include "GeoSceneZoom.h"
-#include "HttpDownloadManager.h"
+#include "MarbleDataFacade.h"
 #include "MarbleDebug.h"
 #include "MarbleDirs.h"
 #include "MarbleLocale.h"
@@ -52,14 +56,18 @@
 #include "PlacemarkLayout.h"
 #include "Planet.h"
 #include "RenderPlugin.h"
-#include "StoragePolicy.h"
 #include "SunLocator.h"
+#include "StackedTileLoader.h"
 #include "TextureColorizer.h"
+#include "TextureLayer.h"
+#include "TileCoordsPyramid.h"
+#include "TileCreator.h"
+#include "TileCreatorDialog.h"
 #include "ViewParams.h"
 #include "ViewportParams.h"
-#include "gps/GpsLayer.h"
 
 using namespace Marble;
+/* TRANSLATOR Marble::MarbleMap */
 
 #ifdef Q_CC_MSVC
 # ifndef KDEWIN_MATH_H
@@ -68,36 +76,49 @@ using namespace Marble;
 #endif
 
 
-MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent )
+MarbleMapPrivate::MarbleMapPrivate( MarbleMap *parent, MarbleModel *model )
         : m_parent( parent ),
-          m_width( 100 ), m_height( 100 ),
-          m_persistentTileCacheLimit( 0 ), // No limit
-          m_volatileTileCacheLimit( 1024*1024*30 ), // 30 MB
+          m_model( model ),
+          m_backgroundVisible( true ),
+          m_layerManager( model->dataFacade(), model->pluginManager(), parent ),
+          m_textureLayer( model->mapThemeManager(), model->downloadManager(), model->sunLocator() ),
+          m_placemarkLayout( model->popSortModel(), model->placemarkSelectionModel(), parent ),
+          m_measureTool( model ),
           m_viewAngle( 110.0 )
 {
+    GeoDataObject *object = static_cast<GeoDataObject*>( model->dataFacade()->treeModel()->index(0, 0, QModelIndex()).internalPointer());
+    GeoDataDocument *document = dynamic_cast<GeoDataDocument*>( object->parent() );
+    m_geometryLayer = new GeometryLayer( document );
+    m_layerManager.addLayer( m_geometryLayer );
 }
 
 void MarbleMapPrivate::construct()
 {
-    // Some point that tackat defined. :-)
-    m_parent->setHome( -9.4, 54.8, 1050 );
-
     m_parent->connect( m_model, SIGNAL( themeChanged( QString ) ),
                        m_parent, SIGNAL( themeChanged( QString ) ) );
     m_parent->connect( m_model, SIGNAL( modelChanged() ),
-                       m_parent, SLOT( updateChangedMap() ) );
+                       m_parent, SIGNAL( repaintNeeded() ) );
 
-    m_justModified = false;
+    m_parent->connect( &m_veccomposer, SIGNAL( datasetLoaded() ),
+                       m_parent, SIGNAL( repaintNeeded() ));
 
-    m_measureTool = new MeasureTool( m_model, m_parent );
+    QObject::connect( m_model->placemarkSelectionModel(), SIGNAL( selectionChanged( QItemSelection, QItemSelection) ),
+                      &m_placemarkLayout,                 SLOT( requestStyleReset() ) );
+    QObject::connect( m_model->popSortModel(),            SIGNAL( layoutChanged() ),
+                      &m_placemarkLayout,                 SLOT( requestStyleReset() ) );
 
-    m_parent->connect( m_model, SIGNAL( timeout() ),
-                       m_parent, SLOT( updateGps() ) );
+    // FIXME: more on the spot update names and API
+    QObject::connect ( &m_layerManager, SIGNAL( floatItemsChanged() ),
+                       m_parent,        SIGNAL( repaintNeeded() ) );
+
+    QObject::connect ( &m_layerManager, SIGNAL( pluginSettingsChanged() ),
+                       m_parent,        SIGNAL( pluginSettingsChanged() ) );
+    QObject::connect ( &m_layerManager, SIGNAL( repaintNeeded( QRegion ) ),
+                       m_parent,        SIGNAL( repaintNeeded( QRegion ) ) );
+    QObject::connect ( &m_layerManager, SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
+                       m_parent,        SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
 
     m_logzoom  = 0;
-    m_zoomStep = 40;
-
-    m_parent->goHome();
 
     // FloatItems
     m_showFrameRate = false;
@@ -108,45 +129,10 @@ void MarbleMapPrivate::construct()
     m_parent->connect( m_model->sunLocator(), SIGNAL( centerSun() ),
                        m_parent,              SLOT( centerSun() ) );
 
-    m_parent->connect( m_model,               SIGNAL( repaintNeeded( QRegion ) ),
-                       m_parent,              SIGNAL( repaintNeeded( QRegion ) ) );
-
-    // A new instance of FileStorageWatcher.
-    // The thread will be started at setting persistent tile cache size.
-    m_storageWatcher = new FileStorageWatcher( MarbleDirs::localPath(), m_parent );
-    m_parent->connect( m_parent, SIGNAL( themeChanged( QString ) ),
-		       m_storageWatcher, SLOT( updateTheme( QString ) ) );
-    // Setting the theme to the current theme.
-    m_storageWatcher->updateTheme( m_parent->mapThemeId() );
-    // connect the StoragePolicy used by the download manager to the FileStorageWatcher
-    StoragePolicy * const storagePolicy = m_model->downloadManager()->storagePolicy();
-    QObject::connect( storagePolicy, SIGNAL( cleared() ),
-                      m_storageWatcher, SLOT( resetCurrentSize() ) );
-    QObject::connect( storagePolicy, SIGNAL( sizeChanged( qint64 ) ),
-                      m_storageWatcher, SLOT( addToCurrentSize( qint64 ) ) );
-}
-
-// Used to be resizeEvent()
-void MarbleMapPrivate::doResize()
-{
-    QSize size( m_parent->width(), m_parent->height() );
-    m_viewParams.viewport()->setSize( size );
-
-    // If the globe covers fully the screen then we can use the faster
-    // RGB32 as there are no translucent areas involved.
-    QImage::Format imageFormat = ( m_parent->mapCoversViewport() )
-                                 ? QImage::Format_RGB32
-                                 : QImage::Format_ARGB32_Premultiplied;
-
-    // Recreate the canvas image with the new size.
-    m_viewParams.setCanvasImage( new QImage( m_parent->width(), m_parent->height(),
-                                             imageFormat ));
-
-    // Recreate the coastline detection offscreen image
-    m_viewParams.setCoastImage( new QImage( m_parent->width(), m_parent->height(),
-                                            QImage::Format_RGB32 ));
-
-    m_justModified = true;
+    m_parent->connect( &m_textureLayer, SIGNAL( tileLevelChanged( int ) ),
+                       m_parent, SIGNAL( tileLevelChanged( int ) ) );
+    m_parent->connect( &m_textureLayer, SIGNAL( repaintNeeded( QRegion ) ),
+                       m_parent, SIGNAL( repaintNeeded( QRegion ) ) );
 }
 
 void  MarbleMapPrivate::paintMarbleSplash( GeoPainter &painter, QRect &dirtyRect )
@@ -188,36 +174,69 @@ void MarbleMapPrivate::paintGround( GeoPainter &painter, QRect &dirtyRect )
         return;
     }
 
-    m_model->paintGlobe( &painter,
-                         m_parent->width(), m_parent->height(), &m_viewParams,
-                         m_parent->needsUpdate() || m_viewParams.canvasImage()->isNull(),
-                         dirtyRect );
-    // FIXME: this is ugly, add method updatePlanetAxis() to ViewParams
-    m_viewParams.setPlanetAxisUpdated( m_viewParams.planetAxis() );
-    // FIXME: this is ugly, add method updateRadius() to ViewParams
-    m_viewParams.setRadiusUpdated( m_viewParams.radius() );
-    m_justModified = false;
-}
+    QStringList renderPositions;
 
-void MarbleMapPrivate::paintOverlay( GeoPainter &painter, QRect &dirtyRect )
-{
-    Q_UNUSED( dirtyRect )
-
-    if ( !m_viewParams.mapTheme() ) {
-        return;
+    if ( m_backgroundVisible ) {
+        renderPositions << "STARS" << "BEHIND_TARGET";
+        m_layerManager.renderLayers( &painter, &m_viewParams, renderPositions );
     }
 
-    // FIXME: Add this stuff into the Layermanager as something to be 
-    // called before the float items.
-
-    bool antialiased = false;
-
-    if (   m_viewParams.mapQuality() == HighQuality
-        || m_viewParams.mapQuality() == PrintQuality ) {
-            antialiased = true;
+    if ( m_viewParams.showAtmosphere() ) {
+        m_atmosphereLayer.render( &painter, m_viewParams.viewport() );
     }
 
-    m_measureTool->paint( &painter, m_viewParams.viewport(), antialiased );
+    m_textureLayer.paintGlobe( &painter, &m_viewParams, dirtyRect );
+
+    renderPositions.clear();
+    renderPositions << "SURFACE";
+
+    // Paint the vector layer.
+    if ( m_model->mapTheme()->map()->hasVectorLayers() ) {
+
+        if ( !m_model->mapTheme()->map()->hasTextureLayers() ) {
+            m_veccomposer.paintBaseVectorMap( &painter, &m_viewParams );
+        }
+
+        m_layerManager.renderLayers( &painter, &m_viewParams, renderPositions );
+
+        // Add further Vectors
+        m_veccomposer.paintVectorMap( &painter, &m_viewParams );
+    }
+    else {
+        m_layerManager.renderLayers( &painter, &m_viewParams, renderPositions );
+    }
+
+    // Paint the GeoDataPlacemark layer
+    bool showPlaces = false;
+    bool showCities = false;
+    bool showTerrain = false;
+    bool showOtherPlaces = false;
+
+    m_viewParams.propertyValue( "places", showPlaces );
+    m_viewParams.propertyValue( "cities", showCities );
+    m_viewParams.propertyValue( "terrain", showTerrain );
+    m_viewParams.propertyValue( "otherplaces", showOtherPlaces );
+
+    if ( showPlaces || showCities || showTerrain || showOtherPlaces )
+    {
+        m_placemarkLayout.paintPlaceFolder( &painter, &m_viewParams );
+    }
+
+    renderPositions.clear();
+    renderPositions << "HOVERS_ABOVE_SURFACE";
+    m_layerManager.renderLayers( &painter, &m_viewParams, renderPositions );
+
+    // FIXME: This is really slow. That's why we defer this to
+    //        PrintQuality. Either cache on a pixmap - or maybe
+    //        better: Add to GlobeScanlineTextureMapper.
+
+    if ( m_viewParams.mapQuality() == PrintQuality )
+        m_fogLayer.render( &painter, m_viewParams.viewport() );
+
+    renderPositions.clear();
+    renderPositions << "ATMOSPHERE"
+                    << "ORBIT" << "ALWAYS_ON_TOP" << "FLOAT_ITEM" << "USER_TOOLS";
+    m_layerManager.renderLayers( &painter, &m_viewParams, renderPositions );
 }
 
 void MarbleMapPrivate::paintFps( GeoPainter &painter, QRect &dirtyRect, qreal fps )
@@ -245,7 +264,7 @@ void MarbleMapPrivate::paintFps( GeoPainter &painter, QRect &dirtyRect, qreal fp
 
 
 MarbleMap::MarbleMap()
-    : d( new MarbleMapPrivate( this ) )
+    : d( new MarbleMapPrivate( this, new MarbleModel( this ) ) )
 {
 #ifdef MARBLE_DBUS
     QDBusConnection::sessionBus().registerObject( "/MarbleMap", this, 
@@ -256,36 +275,30 @@ MarbleMap::MarbleMap()
     QTime t;
     t.start();
     
-    d->m_model        = new MarbleModel( this );
     d->m_modelIsOwned = true;
 
     d->construct();
-    d->m_model->setupVectorComposer();
     qDebug("Model: Time elapsed: %d ms", t.elapsed());
 }
 
 MarbleMap::MarbleMap(MarbleModel *model)
-    : d( new MarbleMapPrivate( this ) )
+    : d( new MarbleMapPrivate( this, model ) )
 {
 //     QDBusConnection::sessionBus().registerObject( "/marble", this,
 //                                                   QDBusConnection::ExportAllSlots );
 
-    d->m_model        = model;
     d->m_modelIsOwned = false;
 
     d->construct();
-    d->m_model->setupVectorComposer();
 }
 
 MarbleMap::~MarbleMap()
 {
-    // Some basic initializations.
-    d->m_width  = 0;
-    d->m_height = 0;
+    MarbleModel *model = d->m_modelIsOwned ? d->m_model : 0;
 
-    if ( d->m_modelIsOwned )
-        delete d->m_model;
     delete d;
+
+    delete model;  // delete the model after private data
 }
 
 MarbleModel *MarbleMap::model() const
@@ -293,48 +306,51 @@ MarbleModel *MarbleMap::model() const
     return d->m_model;
 }
 
-ViewParams *MarbleMap::viewParams()
+ViewportParams *MarbleMap::viewport()
 {
-    return &d->m_viewParams;
+    return d->m_viewParams.viewport();
 }
 
-MeasureTool *MarbleMap::measureTool()
+
+void MarbleMap::setMapQuality( MapQuality quality )
 {
-    return d->m_measureTool;
+    d->m_viewParams.setMapQuality( quality );
+    // Update texture map during the repaint that follows:
+    d->m_textureLayer.setNeedsUpdate();
+}
+
+MapQuality MarbleMap::mapQuality() const
+{
+    return d->m_viewParams.mapQuality();
 }
 
 
 void MarbleMap::setSize( int width, int height )
 {
-    d->m_width  = width;
-    d->m_height = height;
-
-    d->doResize();
+    d->m_viewParams.setSize( width, height );
     emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
+
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setSize( const QSize& size )
 {
-    d->m_width  = size.width();
-    d->m_height = size.height();
-
-    d->doResize();
-    emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
+    setSize( size.width(), size.height() );
 }
 
 QSize MarbleMap::size() const
 {
-    return QSize( d->m_width, d->m_height );
+    return QSize( d->m_viewParams.width(), d->m_viewParams.height() );
 }
 
 int  MarbleMap::width() const
 {
-    return d->m_width;
+    return d->m_viewParams.width();
 }
 
 int  MarbleMap::height() const
 {
-    return d->m_height;
+    return d->m_viewParams.height();
 }
 
 Quaternion MarbleMap::planetAxis() const
@@ -352,9 +368,7 @@ void MarbleMap::setRadius( int radius )
 {
     d->m_viewParams.setRadius( radius );
 
-    if ( !mapCoversViewport() ) {
-        setNeedsUpdate();
-    }
+    d->m_textureLayer.setNeedsUpdate();
 
     d->m_logzoom = qRound( d->zoom( radius ) );
     emit zoomChanged( d->m_logzoom );
@@ -362,18 +376,6 @@ void MarbleMap::setRadius( int radius )
     emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
 }
 
-bool MarbleMap::needsUpdate() const
-{
-    return ( d->m_justModified
-             || d->m_viewParams.radius() != d->m_viewParams.radiusUpdated()
-             || !( d->m_viewParams.planetAxis() == d->m_viewParams.planetAxisUpdated() ) );
-}
-
-void MarbleMap::setNeedsUpdate()
-{
-    d->m_justModified = true;
-}
-
 
 QAbstractItemModel *MarbleMap::placemarkModel() const
 {
@@ -385,20 +387,16 @@ QItemSelectionModel *MarbleMap::placemarkSelectionModel() const
     return d->m_model->placemarkSelectionModel();
 }
 
-qreal MarbleMap::moveStep()
-{
-    if ( radius() < sqrt( (qreal)(width() * width() + height() * height()) ) )
-	return 180.0 * 0.1;
-    else
-	return 180.0 * atan( (qreal)width()
-		     / (qreal)( 2 * radius() ) ) * 0.2;
-}
-
 int MarbleMap::zoom() const
 {
     return d->m_logzoom;
 }
 
+int MarbleMap::tileZoomLevel() const
+{
+    return d->m_textureLayer.tileZoomLevel();
+}
+
 
 qreal MarbleMap::distance() const
 {
@@ -478,26 +476,30 @@ int  MarbleMap::maximumZoom() const
 
 void MarbleMap::addPlacemarkFile( const QString &filename )
 {
-    d->m_model->addPlacemarkFile( filename );
+    addGeoDataFile( filename );
 }
 
 void MarbleMap::addPlacemarkData( const QString &data, const QString &key )
 {
-    d->m_model->addPlacemarkData( data, key );
+    addGeoDataString( data, key );
 }
 
 void MarbleMap::removePlacemarkKey( const QString &key )
 {
-    d->m_model->removePlacemarkKey( key );
+    removeGeoData( key );
 }
 
+QVector<QModelIndex> MarbleMap::whichFeatureAt( const QPoint& curpos ) const
+{
+    return d->m_placemarkLayout.whichPlacemarkAt( curpos );
+}
 
 QPixmap MarbleMap::mapScreenShot()
 {
     QPixmap screenshotPixmap( size() );
     screenshotPixmap.fill( Qt::transparent );
 
-    GeoPainter painter( &screenshotPixmap, viewParams()->viewport(),
+    GeoPainter painter( &screenshotPixmap, viewport(),
                         PrintQuality );
     painter.begin( &screenshotPixmap );
     QRect dirtyRect( QPoint(), size() );
@@ -509,7 +511,54 @@ QPixmap MarbleMap::mapScreenShot()
 
 void MarbleMap::reload() const
 {
-    d->m_model->reloadMap();
+    d->m_textureLayer.reload();
+}
+
+void MarbleMap::downloadRegion( const QString& sourceDir, QVector<TileCoordsPyramid> const & pyramid )
+{
+    Q_ASSERT( textureLayer() );
+    Q_ASSERT( !pyramid.isEmpty() );
+    QTime t;
+    t.start();
+
+    // When downloading a region (the author of these lines thinks) most users probably expect
+    // the download to begin with the low resolution tiles and then procede level-wise to
+    // higher resolution tiles. In order to achieve this, we start requesting downloads of
+    // high resolution tiles and request the low resolution tiles at the end because
+    // DownloadQueueSet (silly name) is implemented as stack.
+
+
+    int const first = 0;
+    int tilesCount = 0;
+
+     for ( int level = pyramid[first].bottomLevel(); level >= pyramid[first].topLevel(); --level ) {
+         QSet<TileId> tileIdSet;
+          for( int i = 0; i < pyramid.size(); ++i ) {
+            QRect const coords = pyramid[i].coords( level );
+            mDebug() << "MarbleMap::downloadRegion level:" << level << "tile coords:" << coords;
+            int x1, y1, x2, y2;
+            coords.getCoords( &x1, &y1, &x2, &y2 );
+            for ( int x = x1; x <= x2; ++x ) {
+                for ( int y = y1; y <= y2; ++y ) {
+                    TileId const tileId( sourceDir, level, x, y );
+                    tileIdSet.insert( tileId );
+                    // FIXME: use lazy evaluation to not generate up to 100k tiles in one go
+                    // this can take considerable time even on very fast systems
+                    // in contrast generating the TileIds on the fly when they are needed
+                    // does not seem to affect download speed.
+                }
+            }
+         }
+         QSetIterator<TileId> i( tileIdSet );
+         while( i.hasNext() ) {
+              textureLayer()->downloadTile( i.next() );
+         }
+         tilesCount += tileIdSet.count();
+     }
+    // Needed for downloading unique tiles only. Much faster than if tiles for each level is downloaded separately
+
+    int const elapsedMs = t.elapsed();
+    mDebug() << "MarbleMap::downloadRegion:" << tilesCount << "tiles, " << elapsedMs << "ms";
 }
 
 bool MarbleMap::propertyValue( const QString& name ) const
@@ -617,7 +666,7 @@ bool MarbleMap::showLakes() const
 
 bool MarbleMap::showGps() const
 {
-    return d->m_model->gpsLayer()->visible();
+    return d->m_viewParams.showGps();
 }
 
 bool MarbleMap::showFrameRate() const
@@ -625,14 +674,19 @@ bool MarbleMap::showFrameRate() const
     return d->m_showFrameRate;
 }
 
+bool MarbleMap::showBackground() const
+{
+    return d->m_backgroundVisible;
+}
+
 quint64 MarbleMap::persistentTileCacheLimit() const
 {
-    return d->m_persistentTileCacheLimit;
+    return d->m_model->persistentTileCacheLimit();
 }
 
 quint64 MarbleMap::volatileTileCacheLimit() const
 {
-    return d->m_volatileTileCacheLimit;
+    return d->m_textureLayer.volatileCacheLimit();
 }
 
 void MarbleMap::zoomView( int newZoom )
@@ -649,7 +703,7 @@ void MarbleMap::zoomView( int newZoom )
 
     d->m_viewParams.setRadius( d->radius( newZoom ) );
 
-    setNeedsUpdate();
+    d->m_textureLayer.setNeedsUpdate();
 
     d->m_logzoom = newZoom;
     emit zoomChanged( d->m_logzoom );
@@ -664,19 +718,11 @@ void MarbleMap::zoomViewBy( int zoomStep )
 }
 
 
-void MarbleMap::zoomIn()
-{
-    zoomViewBy( d->m_zoomStep );
-}
-
-void MarbleMap::zoomOut()
-{
-    zoomViewBy( -d->m_zoomStep );
-}
-
 void MarbleMap::rotateBy( const Quaternion& incRot )
 {
     d->m_viewParams.setPlanetAxis( incRot * d->m_viewParams.planetAxis() );
+    d->m_textureLayer.setNeedsUpdate();
+
     emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
 }
 
@@ -690,6 +736,8 @@ void MarbleMap::rotateBy( const qreal& deltaLon, const qreal& deltaLat )
     axis *= rotPhi;
     axis.normalize();
     d->m_viewParams.setPlanetAxis( axis );
+    d->m_textureLayer.setNeedsUpdate();
+
     emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
 }
 
@@ -699,6 +747,8 @@ void MarbleMap::centerOn( const qreal lon, const qreal lat )
     Quaternion  quat;
     quat.createFromEuler( -lat * DEG2RAD, lon * DEG2RAD, 0.0 );
     d->m_viewParams.setPlanetAxis( quat );
+    d->m_textureLayer.setNeedsUpdate();
+
     emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
 }
 
@@ -745,74 +795,14 @@ void MarbleMap::setProjection( Projection projection )
 
     d->m_viewParams.setProjection( projection );
 
-    d->m_model->setupTextureMapper( projection );
+    d->m_textureLayer.setupTextureMapper( projection );
 
-    // Update texture map during the repaint that follows:
-    setNeedsUpdate();
     emit visibleLatLonAltBoxChanged( d->m_viewParams.viewport()->viewLatLonAltBox() );
 }
 
-void MarbleMap::home( qreal &lon, qreal &lat, int& zoom )
-{
-    d->m_homePoint.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
-    zoom = d->m_homeZoom;
-}
-
-void MarbleMap::setHome( qreal lon, qreal lat, int zoom )
-{
-    d->m_homePoint = GeoDataCoordinates( lon, lat, 0, GeoDataCoordinates::Degree );
-    d->m_homeZoom = zoom;
-}
-
-void MarbleMap::setHome( const GeoDataCoordinates& homePoint, int zoom )
-{
-    d->m_homePoint = homePoint;
-    d->m_homeZoom = zoom;
-}
-
-
-void MarbleMap::moveLeft()
-{
-    int polarity = viewParams()->viewport()->polarity();
-
-    if ( polarity < 0 )
-        rotateBy( +moveStep(), 0 );
-    else
-        rotateBy( -moveStep(), 0 );
-}
-
-void MarbleMap::moveRight()
-{
-    int polarity = viewParams()->viewport()->polarity();
-
-    if ( polarity < 0 )
-        rotateBy( -moveStep(), 0 );
-    else
-        rotateBy( +moveStep(), 0 );
-}
-
-
-void MarbleMap::moveUp()
-{
-    rotateBy( 0, -moveStep() );
-}
-
-void MarbleMap::moveDown()
-{
-    rotateBy( 0, +moveStep() );
-}
-
-int MarbleMap::northPoleY()
-{
-    Quaternion  northPole     = GeoDataCoordinates( 0.0, M_PI * 0.5 ).quaternion();
-    Quaternion  invPlanetAxis = d->m_viewParams.planetAxis().inverse();
-
-    northPole.rotateAroundAxis( invPlanetAxis );
-    return (int)( d->m_viewParams.radius() * northPole.v[Q_Y] );
-}
 
 bool MarbleMap::screenCoordinates( qreal lon, qreal lat,
-                                   qreal& x, qreal& y )
+                                   qreal& x, qreal& y ) const
 {
     return d->m_viewParams.currentProjection()
         ->screenCoordinates( lon * DEG2RAD, lat * DEG2RAD,
@@ -822,7 +812,7 @@ bool MarbleMap::screenCoordinates( qreal lon, qreal lat,
 
 bool MarbleMap::geoCoordinates( int x, int y,
                                 qreal& lon, qreal& lat,
-                                GeoDataCoordinates::Unit unit )
+                                GeoDataCoordinates::Unit unit ) const
 {
     return d->m_viewParams.currentProjection()
         ->geoCoordinates( x, y, d->m_viewParams.viewport(),
@@ -837,7 +827,7 @@ void MarbleMap::paint( GeoPainter &painter, QRect &dirtyRect )
     
     d->paintGround( painter, dirtyRect );
     customPaint( &painter );
-    d->paintOverlay( painter, dirtyRect );
+    d->m_measureTool.render( &painter, viewport() );
 
     qreal fps = 1000.0 / (qreal)( t.elapsed() );
     d->paintFps( painter, dirtyRect, fps );
@@ -848,22 +838,11 @@ void MarbleMap::customPaint( GeoPainter *painter )
 {
     Q_UNUSED( painter );
 
-    if ( !viewParams()->mapTheme() ) {
+    if ( !d->m_viewParams.mapTheme() ) {
         return;
     }
 }
 
-void MarbleMap::goHome()
-{
-    qreal  homeLon = 0;
-    qreal  homeLat = 0;
-    d->m_homePoint.geoCoordinates( homeLon, homeLat );
-
-    centerOn( homeLon * RAD2DEG, homeLat * RAD2DEG );
-
-    zoomView( d->m_homeZoom ); // default 1050
-}
-
 QString MarbleMap::mapThemeId() const
 {
     return d->m_model->mapThemeId();
@@ -877,22 +856,115 @@ void MarbleMap::setMapThemeId( const QString& mapThemeId )
     d->m_viewParams.setMapThemeId( mapThemeId );
     GeoSceneDocument *mapTheme = d->m_viewParams.mapTheme();
 
-    if ( mapTheme ) {
-        d->m_model->setMapTheme( mapTheme, d->m_viewParams.projection() );
-
-        d->doResize();
-
-        centerSun();
-
-        // Update texture map during the repaint that follows:
-        setNeedsUpdate();
+    if ( !mapTheme ) {
+        return;
     }
+
+    // NOTE due to frequent regressions: 
+    // Do NOT take it for granted that there is any TEXTURE or VECTOR data AVAILABLE
+    // at this point. Some themes do NOT have either vector or texture data!
+    
+    // Check whether there is a vector layer available:
+    if ( mapTheme->map()->hasVectorLayers() ) {
+	// Set all the colors for the vector layers
+        d->m_veccomposer.setOceanColor( mapTheme->map()->backgroundColor() );
+
+        // Just as with textures, this is a workaround for DGML2 to
+        // emulate the old behaviour.
+
+        GeoSceneLayer *layer = mapTheme->map()->layer( "mwdbii" );
+        if ( layer ) {
+            GeoSceneVector *vector = 0;
+
+            vector = static_cast<GeoSceneVector*>( layer->dataset("pdiffborder") );
+            if ( vector )
+                d->m_veccomposer.setCountryBorderColor( vector->pen().color() );
+
+            vector = static_cast<GeoSceneVector*>( layer->dataset("rivers") );
+            if ( vector )
+                d->m_veccomposer.setRiverColor( vector->pen().color() );
+
+            vector = static_cast<GeoSceneVector*>( layer->dataset("pusa48") );
+            if ( vector )
+                d->m_veccomposer.setStateBorderColor( vector->pen().color() );
+
+            vector = static_cast<GeoSceneVector*>( layer->dataset("plake") );
+            if ( vector )
+                d->m_veccomposer.setLakeColor( vector->pen().color() );
+
+            vector = static_cast<GeoSceneVector*>( layer->dataset("pcoast") );
+            if ( vector )
+            {
+                d->m_veccomposer.setLandColor( vector->brush().color() );
+                d->m_veccomposer.setCoastColor( vector->pen().color() );
+            }
+        }
+    }
+    
+    // NOTE due to frequent regressions: 
+    // Do NOT take it for granted that there is any TEXTURE or VECTOR data AVAILABLE
+    // at this point.
+
+    // Check whether there is a texture layer available:
+    if ( mapTheme->map()->hasTextureLayers() ) {
+	d->m_textureLayer.setMapTheme( mapTheme );
+        // If the tiles aren't already there, put up a progress dialog
+        // while creating them.
+
+        // As long as we don't have an Layer Management Class we just lookup
+        // the name of the layer that has the same name as the theme ID
+        QString themeID = mapTheme->head()->theme();
+
+        GeoSceneLayer *layer =
+            static_cast<GeoSceneLayer*>( mapTheme->map()->layer( themeID ) );
+        GeoSceneTexture *texture =
+            static_cast<GeoSceneTexture*>( layer->groundDataset() );
+
+        QString sourceDir = texture->sourceDir();
+        QString installMap = texture->installMap();
+        QString role = mapTheme->map()->layer( themeID )->role();
+
+        if ( !StackedTileLoader::baseTilesAvailable( layer )
+            && !installMap.isEmpty() )
+        {
+            mDebug() << "Base tiles not available. Creating Tiles ... \n"
+                     << "SourceDir: " << sourceDir << "InstallMap:" << installMap;
+            MarbleDirs::debug();
+
+            TileCreator *tileCreator = new TileCreator(
+                                     sourceDir,
+                                     installMap,
+                                     (role == "dem") ? "true" : "false" );
+
+            QPointer<TileCreatorDialog> tileCreatorDlg = new TileCreatorDialog( tileCreator, 0 );
+            tileCreatorDlg->setSummary( mapTheme->head()->name(),
+                                        mapTheme->head()->description() );
+            tileCreatorDlg->exec();
+            qDebug("Tile creation completed");
+            delete tileCreatorDlg;
+        }
+
+        d->m_textureLayer.setupTextureMapper( d->m_viewParams.projection() );
+    }
+
+    // NOTE due to frequent regressions: 
+    // Do NOT take it for granted that there is any TEXTURE or VECTOR data AVAILABLE
+    // at this point!
+    
+    d->m_placemarkLayout.requestStyleReset();
+
+    d->m_model->setMapTheme( mapTheme );
+
+    centerSun();
+
+    d->m_layerManager.syncViewParamsAndPlugins( mapTheme );
 }
 
 void MarbleMap::setPropertyValue( const QString& name, bool value )
 {
     mDebug() << "In MarbleMap the property " << name << "was set to " << value;
     d->m_viewParams.setPropertyValue( name, value );
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowOverviewMap( bool visible )
@@ -912,14 +984,7 @@ void MarbleMap::setShowCompass( bool visible )
 
 void MarbleMap::setShowAtmosphere( bool visible )
 {
-    bool previousVisible = showAtmosphere();
-
-    if ( visible != previousVisible ) {
-        d->m_viewParams.setShowAtmosphere( visible );
-
-        // Quick and dirty way to force a whole update of the view
-        d->doResize();
-    }
+    d->m_viewParams.setShowAtmosphere( visible );
 }
 
 void MarbleMap::setShowCrosshairs( bool visible )
@@ -937,12 +1002,12 @@ void MarbleMap::setShowCrosshairs( bool visible )
 void MarbleMap::setShowClouds( bool visible )
 {
     d->m_viewParams.setShowClouds( visible );
-    setNeedsUpdate();
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowTileId( bool visible )
 {
-    d->m_model->layerDecorator()->setShowTileId( visible );
+    d->m_textureLayer.setShowTileId( visible );
 }
 
 void MarbleMap::setShowGrid( bool visible )
@@ -974,21 +1039,21 @@ void MarbleMap::setShowRelief( bool visible )
 {
     setPropertyValue( "relief", visible );
     // Update texture map during the repaint that follows:
-    setNeedsUpdate();
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowElevationModel( bool visible )
 {
     d->m_viewParams.setShowElevationModel( visible );
     // Update texture map during the repaint that follows:
-    setNeedsUpdate();
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowIceLayer( bool visible )
 {
     setPropertyValue( "ice", visible );
     // Update texture map during the repaint that follows:
-    setNeedsUpdate();
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowBorders( bool visible )
@@ -1005,7 +1070,7 @@ void MarbleMap::setShowLakes( bool visible )
 {
     setPropertyValue( "lakes", visible );
     // Update texture map during the repaint that follows:
-    setNeedsUpdate();
+    d->m_textureLayer.setNeedsUpdate();
 }
 
 void MarbleMap::setShowFrameRate( bool visible )
@@ -1013,16 +1078,16 @@ void MarbleMap::setShowFrameRate( bool visible )
     d->m_showFrameRate = visible;
 }
 
+void MarbleMap::setShowBackground( bool visible )
+{
+    d->m_backgroundVisible = visible;
+}
+
 void MarbleMap::setShowGps( bool visible )
 {
     d->m_viewParams.setShowGps( visible );
 }
 
-void MarbleMap::changeCurrentPosition( qreal lon, qreal lat )
-{
-    d->m_model->gpsLayer()->changeCurrentPosition( lat, lon );
-}
-
 void MarbleMap::notifyMouseClick( int x, int y )
 {
     qreal  lon   = 0;
@@ -1035,26 +1100,9 @@ void MarbleMap::notifyMouseClick( int x, int y )
     }
 }
 
-void MarbleMap::updateGps()
-{
-    QRegion temp;
-//    const bool draw = 
-    d->m_model->gpsLayer()->updateGps( size(),&d->m_viewParams, temp );
-#if 0  // FIXME: move to MarbleWidget?
-    if ( draw ) {
-        update(temp);
-    }
-#endif
-}
-
 void MarbleMap::openGpxFile( const QString &filename )
 {
-    d->m_model->openGpxFile( filename );
-}
-
-GpxFileModel *MarbleMap::gpxFileModel()
-{
-    return d->m_model->gpxFileModel();
+    addGeoDataFile( filename );
 }
 
 FileViewModel* MarbleMap::fileViewModel() const
@@ -1069,37 +1117,19 @@ void MarbleMap::clearPersistentTileCache()
 
 void MarbleMap::setPersistentTileCacheLimit( quint64 kiloBytes )
 {
-    d->m_persistentTileCacheLimit = kiloBytes;
-    d->m_storageWatcher->setCacheLimit( kiloBytes * 1024 );
-    
-    if( kiloBytes != 0 )
-    {
-	if( !d->m_storageWatcher->isRunning() )
-	    d->m_storageWatcher->start( QThread::IdlePriority );
-    }
-    else
-    {
-	d->m_storageWatcher->quit();
-    }
-    // TODO: trigger update
+    d->m_model->setPersistentTileCacheLimit( kiloBytes );
 }
 
 void MarbleMap::clearVolatileTileCache()
 {
-    d->m_model->clearVolatileTileCache();
+    d->m_textureLayer.update();
+    mDebug() << "Cleared Volatile Cache!";
 }
 
 void MarbleMap::setVolatileTileCacheLimit( quint64 kilobytes )
 {
     mDebug() << "kiloBytes" << kilobytes;
-    d->m_volatileTileCacheLimit = kilobytes;
-    d->m_model->setVolatileTileCacheLimit( kilobytes );
-}
-
-void MarbleMap::updateChangedMap()
-{
-    // Update texture map during the repaint that follows:
-    setNeedsUpdate();
+    d->m_textureLayer.setVolatileCacheLimit( kilobytes );
 }
 
 QString MarbleMap::distanceString() const
@@ -1153,7 +1183,7 @@ QFont MarbleMap::defaultFont() const
 void MarbleMap::setDefaultFont( const QFont& font )
 {
     GeoDataFeature::setDefaultFont( font );
-    d->m_model->placemarkLayout()->requestStyleReset();
+    d->m_placemarkLayout.requestStyleReset();
 }
 
 void MarbleMap::updateSun()
@@ -1162,8 +1192,8 @@ void MarbleMap::updateSun()
     //SunLocator  *sunLocator = d->m_model->sunLocator();
 
     mDebug() << "MarbleMap: Updating the sun shading map...";
-    d->m_model->update();
-    setNeedsUpdate();
+    d->m_textureLayer.update();
+    d->m_textureLayer.setNeedsUpdate();
     //mDebug() << "Finished updating the sun shading map";
 }
 
@@ -1187,20 +1217,52 @@ SunLocator* MarbleMap::sunLocator()
 
 QList<RenderPlugin *> MarbleMap::renderPlugins() const
 {
-    return d->m_model->renderPlugins();
+    return d->m_layerManager.renderPlugins();
 }
 
 QList<AbstractFloatItem *> MarbleMap::floatItems() const
 {
-    return d->m_model->floatItems();
+    return d->m_layerManager.floatItems();
+}
+
+AbstractFloatItem * MarbleMap::floatItem( const QString &nameId ) const
+{
+    foreach ( AbstractFloatItem * floatItem, floatItems() ) {
+        if ( floatItem && floatItem->nameId() == nameId ) {
+            return floatItem;
+        }
+    }
+
+    return 0; // No item found
+}
+
+QList<AbstractDataPlugin *> MarbleMap::dataPlugins()  const
+{
+    return d->m_layerManager.dataPlugins();
+}
+
+QList<AbstractDataPluginItem *> MarbleMap::whichItemAt( const QPoint& curpos ) const
+{
+    return d->m_layerManager.whichItemAt( curpos );
+}
+
+void MarbleMap::addLayer( LayerInterface *layer )
+{
+    d->m_layerManager.addLayer(layer);
+}
+
+void MarbleMap::removeLayer( LayerInterface *layer )
+{
+    d->m_layerManager.removeLayer(layer);
+}
+
+MeasureTool *MarbleMap::measureTool()
+{
+    return &d->m_measureTool;
 }
 
 void MarbleMap::flyTo( const GeoDataLookAt &lookAt )
 {
-    int zoom = zoomFromDistance( lookAt.range() * METER2KM );
-    if ( zoom < minimumZoom() || zoom > maximumZoom() )
-        return; // avoid moving when zooming is impossible
-
     setDistance( lookAt.range() * METER2KM );
     GeoDataCoordinates::Unit deg = GeoDataCoordinates::Degree;
     centerOn( lookAt.longitude( deg ), lookAt.latitude( deg ) );
@@ -1230,5 +1292,25 @@ qreal MarbleMap::zoomFromDistance( qreal distance ) const
     return d->zoom( radiusFromDistance( distance ) );
 }
 
+void MarbleMap::addGeoDataFile( const QString &filename )
+{
+    d->m_model->addGeoDataFile( filename );
+}
+
+void MarbleMap::addGeoDataString( const QString& data, const QString& key )
+{
+    d->m_model->addGeoDataString( data, key );
+}
+
+void MarbleMap::removeGeoData( const QString& key )
+{
+    d->m_model->removeGeoData( key );
+}
+
+// this method will only temporarily "pollute" the MarbleModel class
+TextureLayer* MarbleMap::textureLayer()
+{
+    return &d->m_textureLayer;
+}
 
 #include "MarbleMap.moc"
