@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -41,14 +41,16 @@
 #include "MarblePhysics.h"
 #include "MarblePlacemarkModel.h"
 #include "MarbleWidgetInputHandler.h"
+#include "MeasureTool.h"
 #include "MergedLayerDecorator.h"
 #include "RenderPlugin.h"
 #include "SunLocator.h"
 #include "TileCreatorDialog.h"
-#include "ViewParams.h"
 #include "ViewportParams.h"
-#include "gps/GpsLayer.h"
-
+#include "BookmarkManager.h"
+#include "routing/RoutingLayer.h"
+#include "routing/RoutingManager.h"
+#include "routing/AlternativeRoutesModel.h"
 
 namespace Marble
 {
@@ -73,13 +75,11 @@ class MarbleWidgetPrivate
           m_stillQuality( HighQuality ),
           m_animationQuality( LowQuality ),
           m_animationsEnabled( false ),
+          m_zoomStep( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ? 60 : 40 ),
           m_inputhandler( 0 ),
           m_physics( new MarblePhysics( parent ) ),
-          m_proxyHost(),
-          m_proxyPort( 0 ),
-          m_user(),
-          m_password(),
-          m_repaintTimer()
+          m_repaintTimer(),
+          m_routingLayer( 0 )
     {
     }
 
@@ -96,7 +96,7 @@ class MarbleWidgetPrivate
       * @param stepsDown Number of steps to go down. Negative values go up.
       * @param mode Interpolation mode to use when traveling to the target
       */
-    void moveByStep( MarbleWidget* widget, int stepsRight, int stepsDown, FlyToMode mode );
+    void moveByStep( int stepsRight, int stepsDown, FlyToMode mode );
 
     /**
       * @brief Update widget flags and cause a full repaint
@@ -118,22 +118,16 @@ class MarbleWidgetPrivate
 
     bool m_animationsEnabled;
 
-    // Some values from m_map, as they were last time we repainted.
-    // To store them here will save some repaintings.
-    int              m_logZoom;
+    int              m_zoomStep;
 
     MarbleWidgetInputHandler  *m_inputhandler;
 
     MarblePhysics    *m_physics;
 
-    //This stuff is NEVER used. Needs to be deleted
-    QString          m_proxyHost;
-    qint16           m_proxyPort;
-    QString          m_user;
-    QString          m_password;
-
     // For scheduling repaints
     QTimer           m_repaintTimer;
+
+    RoutingLayer     *m_routingLayer;
 };
 
 
@@ -187,7 +181,7 @@ void MarbleWidgetPrivate::construct()
 
     // Initialize the map and forward some signals.
     m_map->setSize( m_widget->width(), m_widget->height() );
-    m_map->viewParams()->setMapQuality( m_stillQuality );
+    m_map->setMapQuality( m_stillQuality );
 
     m_widget->connect( m_map,    SIGNAL( projectionChanged( Projection ) ),
                        m_widget, SIGNAL( projectionChanged( Projection ) ) );
@@ -196,8 +190,8 @@ void MarbleWidgetPrivate::construct()
     // show this in the view, i.e. here.
     m_widget->connect( m_model,  SIGNAL( themeChanged( QString ) ),
 		       m_widget, SIGNAL( themeChanged( QString ) ) );
-    m_widget->connect( m_model,  SIGNAL( modelChanged() ),
-		       m_widget, SLOT( updateChangedMap() ) );
+    m_widget->connect( m_model, SIGNAL( modelChanged() ),
+                       m_widget, SLOT( update() ) );
 
     // Repaint scheduling
     m_widget->connect( m_map,    SIGNAL( repaintNeeded( QRegion ) ),
@@ -215,6 +209,13 @@ void MarbleWidgetPrivate::construct()
                        m_widget, SIGNAL( distanceChanged( QString ) ) );
     m_widget->connect( m_map,    SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox )),
                        m_widget, SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox )));
+    m_widget->connect( m_map,    SIGNAL( tileLevelChanged( int ) ),
+                       m_widget, SIGNAL( tileLevelChanged( int ) ) );
+
+    m_widget->connect( m_map,    SIGNAL( pluginSettingsChanged() ),
+                       m_widget, SIGNAL( pluginSettingsChanged() ) );
+    m_widget->connect( m_map,    SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
+                       m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
 
     // Set background: black.
     m_widget->setPalette( QPalette ( Qt::black ) );
@@ -223,49 +224,42 @@ void MarbleWidgetPrivate::construct()
     // displayed on the widget background.
     m_widget->setAutoFillBackground( true );
 
-    // Track the GPS current point at timely intervals.
-    m_widget->connect( m_model, SIGNAL( timeout() ),
-                       m_widget, SLOT( updateGps() ) );
-
     // Show a progress dialog when the model calculates new map tiles.
     m_widget->connect( m_model, SIGNAL( creatingTilesStart( TileCreator*, const QString&,
                                                             const QString& ) ),
                        m_widget, SLOT( creatingTilesStart( TileCreator*, const QString&,
                                                            const QString& ) ) );
 
-    m_logZoom  = 0;
-
     m_widget->connect( m_model->sunLocator(), SIGNAL( enableWidgetInput( bool ) ),
                        m_widget, SLOT( setInputEnabled( bool ) ) );
 
     m_widget->connect( m_model->sunLocator(), SIGNAL( updateStars() ),
                        m_widget, SLOT( update() ) );
 
-    m_widget->connect( m_physics, SIGNAL( positionReached( GeoDataLookAt ) ),
-                       m_widget, SLOT( updateAnimation( GeoDataLookAt ) ) );
-
-    m_widget->connect( m_physics, SIGNAL( finished() ),
-                       m_widget, SLOT( startStillMode() ) );
-
     m_widget->connect( m_model->sunLocator(), SIGNAL( centerSun() ),
                        m_widget, SLOT( centerSun() ) );
 
     m_widget->setInputHandler( new MarbleWidgetDefaultInputHandler );
     m_widget->setMouseTracking( m_widget );
 
-    m_widget->connect( m_model, SIGNAL( pluginSettingsChanged() ),
-                       m_widget, SIGNAL( pluginSettingsChanged() ) );
-                       
-    m_widget->connect( m_model, SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
-                       m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
+    m_routingLayer = new RoutingLayer( m_widget, m_widget );
+    m_routingLayer->setRouteRequest( m_model->routingManager()->routeRequest() );
+    m_routingLayer->setModel( m_model->routingManager()->routingModel() );
+    m_map->addLayer( m_routingLayer );
+
+    m_widget->connect( m_routingLayer, SIGNAL( routeDirty() ),
+                       m_model->routingManager(), SLOT( updateRoute() ) );
+    m_widget->connect( m_model->routingManager()->alternativeRoutesModel(),
+                       SIGNAL( currentRouteChanged( GeoDataDocument* ) ),
+                       m_widget, SLOT( repaint() ) );
 }
 
-void MarbleWidgetPrivate::moveByStep( MarbleWidget* widget, int stepsRight, int stepsDown, FlyToMode mode )
+void MarbleWidgetPrivate::moveByStep( int stepsRight, int stepsDown, FlyToMode mode )
 {
-    int polarity = m_map->viewParams()->viewport()->polarity();
-    qreal left = polarity * stepsRight * m_map->moveStep();
-    qreal down = stepsDown * m_map->moveStep();
-    widget->rotateBy( left, down, mode );
+    int polarity = m_widget->viewport()->polarity();
+    qreal left = polarity * stepsRight * m_widget->moveStep();
+    qreal down = stepsDown * m_widget->moveStep();
+    m_widget->rotateBy( left, down, mode );
 }
 
 void MarbleWidgetPrivate::repaint()
@@ -273,7 +267,7 @@ void MarbleWidgetPrivate::repaint()
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
     m_widget->setAttribute( Qt::WA_NoSystemBackground,
-                  m_map->mapCoversViewport() && !m_model->mapThemeId().isEmpty() );
+                  m_widget->viewport()->mapCoversViewport() && !m_model->mapThemeId().isEmpty() );
 
     m_widget->repaint();
 }
@@ -292,6 +286,17 @@ MarbleModel *MarbleWidget::model() const
 }
 
 
+ViewportParams* MarbleWidget::viewport()
+{
+    return d->m_map->viewport();
+}
+
+const ViewportParams* MarbleWidget::viewport() const
+{
+    return d->m_map->viewport();
+}
+
+
 void MarbleWidget::setInputHandler( MarbleWidgetInputHandler *handler )
 {
     delete d->m_inputhandler;
@@ -316,7 +321,7 @@ MarbleWidgetInputHandler *MarbleWidget::inputHandler() const
 
 Quaternion MarbleWidget::planetAxis() const
 {
-    return d->m_map->planetAxis();
+    return viewport()->planetAxis();
 }
 
 
@@ -336,17 +341,6 @@ void MarbleWidget::setRadius( int radius )
 }
 
 
-bool MarbleWidget::needsUpdate() const
-{
-    return d->m_map->needsUpdate();
-}
-
-void MarbleWidget::setNeedsUpdate()
-{
-    d->m_map->setNeedsUpdate();
-}
-
-
 QAbstractItemModel *MarbleWidget::placemarkModel() const
 {
     return d->m_map->placemarkModel();
@@ -359,7 +353,11 @@ QItemSelectionModel *MarbleWidget::placemarkSelectionModel() const
 
 qreal MarbleWidget::moveStep()
 {
-    return d->m_map->moveStep();
+    if ( radius() < sqrt( (qreal)(width() * width() + height() * height()) ) )
+        return 180.0 * 0.1;
+    else
+        return 180.0 * atan( (qreal)width()
+                     / (qreal)( 2 * radius() ) ) * 0.2;
 }
 
 int MarbleWidget::zoom() const
@@ -367,6 +365,11 @@ int MarbleWidget::zoom() const
     return d->m_map->zoom();
 }
 
+int MarbleWidget::tileZoomLevel() const
+{
+    return d->m_map->tileZoomLevel();
+}
+
 int  MarbleWidget::minimumZoom() const
 {
     return d->m_map->minimumZoom();
@@ -379,18 +382,42 @@ int  MarbleWidget::maximumZoom() const
 
 void MarbleWidget::addPlacemarkFile( const QString &filename )
 {
-    d->m_map->addPlacemarkFile( filename );
-    //d->m_model->addPlacemarkFile( filename );
+    addGeoDataFile( filename );
 }
 
 void MarbleWidget::addPlacemarkData( const QString &data, const QString &key )
 {
-    d->m_map->addPlacemarkData( data, key );
+    addGeoDataString( data, key );
 }
 
 void MarbleWidget::removePlacemarkKey( const QString &key )
 {
-    d->m_map->removePlacemarkKey( key );
+    removeGeoData( key );
+}
+
+QVector<QModelIndex> MarbleWidget::whichFeatureAt( const QPoint &curpos ) const
+{
+    return d->m_map->whichFeatureAt( curpos );
+}
+
+QList<AbstractDataPluginItem*> MarbleWidget::whichItemAt( const QPoint &curpos ) const
+{
+    return d->m_map->whichItemAt( curpos );
+}
+
+TextureLayer *MarbleWidget::textureLayer()
+{
+    return d->m_map->textureLayer();
+}
+
+const Marble::TextureLayer* MarbleWidget::textureLayer() const
+{
+    return d->m_map->textureLayer();
+}
+
+MeasureTool *MarbleWidget::measureTool()
+{
+    return d->m_map->measureTool();
 }
 
 QPixmap MarbleWidget::mapScreenShot()
@@ -506,27 +533,16 @@ quint64 MarbleWidget::volatileTileCacheLimit() const
 
 void MarbleWidget::zoomView( int newZoom, FlyToMode mode )
 {
-    if ( mode == Instant || !d->m_animationsEnabled ) {
-        // This function is tricky since it needs to be possible to call
-        // both from above as an ordinary function, and "from below",
-        // i.e. as a slot.  That's why we need to save m_logZoom from when
-        // we repainted last time.
-
-        // Make all the internal changes to the map.
+    // It won't fly anyway. So we should do everything to keep the zoom value.
+    if ( !d->m_animationsEnabled || mode == Instant ) {
         d->m_map->zoomView( newZoom );
-
-        // If no change, we don't need to repainting or anything else.
-        if ( d->m_logZoom == newZoom )
-            return;
-
-        d->m_logZoom = newZoom;
         d->repaint();
     }
     else {
-        GeoDataLookAt target = d->m_map->lookAt();
-        int radius = d->m_map->d->radius( newZoom );
-        target.setRange( 1000 * d->m_map->distanceFromRadius( radius ) );
-        flyTo( target, mode == Automatic ? Instant : mode );
+        GeoDataLookAt target = lookAt();
+        target.setRange( KM2METER * d->m_map->distanceFromZoom( newZoom ) );
+
+        flyTo( target, mode );
     }
 }
 
@@ -539,32 +555,12 @@ void MarbleWidget::zoomViewBy( int zoomStep, FlyToMode mode )
 
 void MarbleWidget::zoomIn( FlyToMode mode )
 {
-    if ( mode == Instant || !d->m_animationsEnabled ) {
-        d->m_map->zoomIn();
-        d->repaint();
-    }
-    else {
-        GeoDataLookAt target = d->m_map->lookAt();
-        MarbleMap *map = d->m_map;
-        int newRadius = map->d->radius( map->zoom() + map->d->m_zoomStep );
-        target.setRange( 1000 * d->m_map->distanceFromRadius( newRadius ) );
-        flyTo( target, mode );
-    }
+    zoomViewBy( d->m_zoomStep, mode );
 }
 
 void MarbleWidget::zoomOut( FlyToMode mode )
 {
-    if ( mode == Instant || !d->m_animationsEnabled ) {
-        d->m_map->zoomOut();
-        d->repaint();
-    }
-    else {
-        GeoDataLookAt target = d->m_map->lookAt();
-        MarbleMap *map = d->m_map;
-        int newRadius = map->d->radius( map->zoom() - map->d->m_zoomStep );
-        target.setRange( 1000 * d->m_map->distanceFromRadius( newRadius ) );
-        flyTo( target, mode );
-    }
+    zoomViewBy( -d->m_zoomStep, mode );
 }
 
 void MarbleWidget::rotateBy( const Quaternion& incRot )
@@ -578,7 +574,7 @@ void MarbleWidget::rotateBy( const qreal deltaLon, const qreal deltaLat, FlyToMo
     Quaternion  rotPhi( 1.0, deltaLat / 180.0, 0.0, 0.0 );
     Quaternion  rotTheta( 1.0, 0.0, deltaLon / 180.0, 0.0 );
 
-    Quaternion  axis = d->m_map->viewParams()->planetAxis();
+    Quaternion  axis = planetAxis();
     qreal lon( 0.0 ), lat( 0.0 );
     axis.getSpherical( lon, lat );
     axis = rotTheta * axis;
@@ -587,7 +583,7 @@ void MarbleWidget::rotateBy( const qreal deltaLon, const qreal deltaLat, FlyToMo
     lat = -axis.pitch();
     lon = axis.yaw();
     
-    GeoDataLookAt target = d->m_map->lookAt();
+    GeoDataLookAt target = lookAt();
     target.setLongitude( lon );
     target.setLatitude( lat );
     flyTo( target, mode );
@@ -602,7 +598,7 @@ void MarbleWidget::centerOn( const qreal lon, const qreal lat, bool animated )
 
 void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
 {
-    QItemSelectionModel *selectionModel = d->m_map->model()->placemarkSelectionModel();
+    QItemSelectionModel *selectionModel = d->m_model->placemarkSelectionModel();
     Q_ASSERT( selectionModel );
 
     selectionModel->clear();
@@ -611,7 +607,7 @@ void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
         const GeoDataCoordinates targetPosition =
             index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
 
-        GeoDataLookAt target = d->m_map->lookAt();
+        GeoDataLookAt target = lookAt();
         target.setLongitude( targetPosition.longitude() );
         target.setLatitude( targetPosition.latitude() );
         flyTo( target, animated ? Automatic : Instant );
@@ -622,7 +618,7 @@ void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
 
 void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 {
-    GeoDataLookAt target = d->m_map->lookAt();
+    GeoDataLookAt target = lookAt();
     target.setLongitude( position.longitude() );
     target.setLatitude( position.latitude() );
     flyTo( target, animated ? Automatic : Instant );
@@ -630,9 +626,7 @@ void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 
 void MarbleWidget::centerOn( const GeoDataLatLonBox &box, bool animated )
 {
-    Q_UNUSED( animated );
-
-    ViewportParams* viewparams = d->m_map->viewParams()->viewport();
+    ViewportParams* viewparams = viewport();
     //prevent divide by zero
     if( box.height() && box.width() ) {
         //work out the needed zoom level
@@ -642,8 +636,9 @@ void MarbleWidget::centerOn( const GeoDataLatLonBox &box, bool animated )
     }
 
     //move the map
-    d->m_map->centerOn( box.center().longitude( GeoDataCoordinates::Degree ),
-                        box.center().latitude( GeoDataCoordinates::Degree ) );
+    centerOn( box.center().longitude( GeoDataCoordinates::Degree ),
+              box.center().latitude( GeoDataCoordinates::Degree ),
+              animated );
 
     repaint();
 }
@@ -676,39 +671,39 @@ void MarbleWidget::setProjection( int projection )
 
 void MarbleWidget::home( qreal &lon, qreal &lat, int& zoom )
 {
-    d->m_map->home( lon, lat, zoom );
+    d->m_model->home( lon, lat, zoom );
 }
 
 void MarbleWidget::setHome( qreal lon, qreal lat, int zoom )
 {
-    d->m_map->setHome( lon, lat, zoom );
+    d->m_model->setHome( lon, lat, zoom );
 }
 
 void MarbleWidget::setHome( const GeoDataCoordinates& homePoint, int zoom )
 {
-    d->m_map->setHome( homePoint, zoom );
+    d->m_model->setHome( homePoint, zoom );
 }
 
 
 void MarbleWidget::moveLeft( FlyToMode mode )
 {
-    d->moveByStep( this, -1, 0, mode );
+    d->moveByStep( -1, 0, mode );
 }
 
 void MarbleWidget::moveRight( FlyToMode mode )
 {
-    d->moveByStep( this, 1, 0, mode );
+    d->moveByStep( 1, 0, mode );
 }
 
 
 void MarbleWidget::moveUp( FlyToMode mode )
 {
-    d->moveByStep( this, 0, -1, mode );
+    d->moveByStep( 0, -1, mode );
 }
 
 void MarbleWidget::moveDown( FlyToMode mode )
 {
-    d->moveByStep( this, 0, 1, mode );
+    d->moveByStep( 0, 1, mode );
 }
 
 void MarbleWidget::leaveEvent( QEvent* )
@@ -740,11 +735,6 @@ void MarbleWidget::disconnectNotify( const char * signal )
             d->m_inputhandler->setPositionSignalConnected( false );
 }
 
-int MarbleWidget::northPoleY()
-{
-    return d->m_map->northPoleY();
-}
-
 bool MarbleWidget::screenCoordinates( qreal lon, qreal lat,
                                       qreal& x, qreal& y )
 {
@@ -770,12 +760,12 @@ qreal MarbleWidget::centerLongitude() const
 
 QRegion MarbleWidget::activeRegion()
 {
-    return d->m_map->viewParams()->viewport()->activeRegion();
+    return viewport()->activeRegion();
 }
 
 QRegion MarbleWidget::mapRegion()
 {
-    return d->m_map->viewParams()->currentProjection()->mapRegion( d->m_map->viewParams()->viewport() );
+    return viewport()->currentProjection()->mapRegion( viewport() );
 }
 
 void MarbleWidget::paintEvent( QPaintEvent *evt )
@@ -787,9 +777,9 @@ void MarbleWidget::paintEvent( QPaintEvent *evt )
 
     // FIXME: Better way to get the GeoPainter
     bool  doClip = true;
-    if ( d->m_map->projection() == Spherical )
-        doClip = ( d->m_map->radius() > width() / 2
-                   || d->m_map->radius() > height() / 2 );
+    if ( projection() == Spherical )
+        doClip = ( radius() > width() / 2
+                   || radius() > height() / 2 );
 
     QPaintDevice *paintDevice = this;
     QImage image;
@@ -802,20 +792,20 @@ void MarbleWidget::paintEvent( QPaintEvent *evt )
                                      : QImage::Format_ARGB32_Premultiplied;
         // Paint to an intermediate image
         image = QImage( rect().size(), imageFormat );
+        image.fill( Qt::transparent );
         paintDevice = &image;
     }
 
     // Create a painter that will do the painting.
-    GeoPainter painter( paintDevice, map()->viewParams()->viewport(),
-                        map()->viewParams()->mapQuality(), doClip );
-
+    GeoPainter painter( paintDevice, map()->viewport(),
+                        map()->mapQuality(), doClip );
     QRect  dirtyRect = evt->rect();
 
     // Draws the map like MarbleMap::paint does, but adds our customPaint in between
     d->m_map->d->paintGround( painter, dirtyRect );
     d->m_map->customPaint( &painter );
     customPaint( &painter );
-    d->m_map->d->paintOverlay( painter, dirtyRect );
+    d->m_map->measureTool()->render( &painter, viewport() );
 
     if ( !isEnabled() )
     {
@@ -826,12 +816,12 @@ void MarbleWidget::paintEvent( QPaintEvent *evt )
             *pixel = qRgb( gray, gray, gray );
         }
 
-        GeoPainter widgetPainter( this, map()->viewParams()->viewport(),
-                            map()->viewParams()->mapQuality(), doClip );
+        GeoPainter widgetPainter( this, map()->viewport(),
+                            map()->mapQuality(), doClip );
         widgetPainter.drawImage( rect(), image );
     }
 
-    if ( d->m_map->showFrameRate() )
+    if ( showFrameRate() )
     {
         qreal fps = 1000.0 / (qreal)( t.elapsed() + 1 );
         d->m_map->d->paintFps( painter, dirtyRect, fps );
@@ -848,27 +838,17 @@ void MarbleWidget::customPaint( GeoPainter *painter )
 
 void MarbleWidget::goHome( FlyToMode mode )
 {
-    if ( !d->m_animationsEnabled || mode == Instant )
-    {
-        d->m_map->goHome();
+    qreal  homeLon = 0;
+    qreal  homeLat = 0;
+    int homeZoom = 0;
+    d->m_model->home( homeLon, homeLat, homeZoom );
 
-         // not obsolete in case the zoomlevel stays unaltered
-        d->repaint();
-    }
-    else {
-        qreal  homeLon = 0;
-        qreal  homeLat = 0;
-        int homeZoom = 0;
-        d->m_map->home( homeLon, homeLat, homeZoom );
+    GeoDataLookAt target;
+    target.setLongitude( homeLon, GeoDataCoordinates::Degree );
+    target.setLatitude( homeLat, GeoDataCoordinates::Degree );
+    target.setRange( 1000 * d->m_map->distanceFromZoom( homeZoom ) );
 
-        GeoDataLookAt target;
-        target.setLongitude( homeLon, GeoDataCoordinates::Degree );
-        target.setLatitude( homeLat, GeoDataCoordinates::Degree );
-        int radius = d->m_map->d->radius( homeZoom );
-        target.setRange( 1000 * d->m_map->distanceFromRadius( radius ) );
-
-        flyTo( target, mode );
-    }
+    flyTo( target, mode );
 }
 
 QString MarbleWidget::mapThemeId() const
@@ -882,9 +862,6 @@ void MarbleWidget::setMapThemeId( const QString& mapThemeId )
         return;
     
     d->m_map->setMapThemeId( mapThemeId );
-    
-    // Update texture map during the repaint that follows:
-    setNeedsUpdate();
 
     // Now we want a full repaint as the atmosphere might differ
     setAttribute( Qt::WA_NoSystemBackground,
@@ -897,7 +874,7 @@ void MarbleWidget::setMapThemeId( const QString& mapThemeId )
 
 GeoSceneDocument *MarbleWidget::mapTheme() const
 {
-    return d->m_map->viewParams()->mapTheme();
+    return d->m_model->mapTheme();
 }
 
 void MarbleWidget::setPropertyValue( const QString& name, bool value )
@@ -905,8 +882,6 @@ void MarbleWidget::setPropertyValue( const QString& name, bool value )
     mDebug() << "In MarbleWidget the property " << name << "was set to " << value;
     d->m_map->setPropertyValue( name, value );
 
-    // Update texture map during the repaint that follows:
-    setNeedsUpdate();
     repaint();
 }
 
@@ -1045,13 +1020,7 @@ void MarbleWidget::setShowGps( bool visible )
 
 void MarbleWidget::setShowTileId( bool visible )
 {
-    d->m_model->layerDecorator()->setShowTileId( visible );
-}
-
-void MarbleWidget::changeCurrentPosition( qreal lon, qreal lat)
-{
-    d->m_model->gpsLayer()->changeCurrentPosition( lat, lon );
-    repaint();
+    d->m_map->setShowTileId( visible );
 }
 
 void MarbleWidget::notifyMouseClick( int x, int y)
@@ -1066,26 +1035,9 @@ void MarbleWidget::notifyMouseClick( int x, int y)
     }
 }
 
-void MarbleWidget::updateGps()
-{
-    QRegion temp;
-    const bool draw = d->m_model->gpsLayer()->updateGps( size(),
-                                                         d->m_map->viewParams(),
-                                                         temp );
-    if ( draw ) {
-//        mDebug() << "Updating viewport for GPS";
-        update( temp );
-    }
-}
-
 void MarbleWidget::openGpxFile( const QString &filename )
 {
-    d->m_map->openGpxFile( filename );
-}
-
-GpxFileModel *MarbleWidget::gpxFileModel()
-{
-    return d->m_model->gpxFileModel();
+    addGeoDataFile( filename );
 }
 
 FileViewModel* MarbleWidget::fileViewModel() const
@@ -1125,13 +1077,6 @@ void MarbleWidget::creatingTilesStart( TileCreator *creator,
     dlg.exec();
 }
 
-void MarbleWidget::updateChangedMap()
-{
-    // Update texture map during the repaint that follows:
-    setNeedsUpdate();
-    update();
-}
-
 void MarbleWidget::scheduleRepaint( const QRegion& dirtyRegion )
 {
     Q_UNUSED( dirtyRegion );
@@ -1149,23 +1094,28 @@ MapQuality MarbleWidget::mapQuality( ViewContext viewContext )
     return d->m_animationQuality; 
 }
 
-void MarbleWidget::setMapQuality( MapQuality mapQuality, ViewContext changedViewContext )
+void MarbleWidget::setMapQuality( MapQuality quality, ViewContext changedViewContext )
 {
+    const MapQuality oldQuality = mapQuality( viewContext() );
+
     // FIXME: Rewrite as a switch
     if ( changedViewContext == Still ) {
-        d->m_stillQuality = mapQuality;
+        d->m_stillQuality = quality;
     }
     else if ( changedViewContext == Animation ) {
-        d->m_animationQuality = mapQuality;
+        d->m_animationQuality = quality;
     }
 
     if ( viewContext() == Still ) {
-        map()->viewParams()->setMapQuality( d->m_stillQuality ); 
+        map()->setMapQuality( d->m_stillQuality ); 
     }
     else if ( viewContext() == Animation )
     {
-        map()->viewParams()->setMapQuality( d->m_animationQuality ); 
+        map()->setMapQuality( d->m_animationQuality ); 
     }
+
+    if ( mapQuality( viewContext() ) != oldQuality )
+        d->repaint();
 }
 
 ViewContext MarbleWidget::viewContext() const
@@ -1175,12 +1125,18 @@ ViewContext MarbleWidget::viewContext() const
 
 void MarbleWidget::setViewContext( ViewContext viewContext )
 {
+    if ( d->m_viewContext == viewContext )
+        return;
+
     d->m_viewContext = viewContext;
 
     if ( viewContext == Still )
-        map()->viewParams()->setMapQuality( d->m_stillQuality ); 
+        map()->setMapQuality( d->m_stillQuality ); 
     if ( viewContext == Animation )
-        map()->viewParams()->setMapQuality( d->m_animationQuality ); 
+        map()->setMapQuality( d->m_animationQuality ); 
+
+    if ( mapQuality( viewContext ) != mapQuality( Animation ) )
+        d->repaint();
 }
 
 bool MarbleWidget::animationsEnabled() const
@@ -1220,8 +1176,8 @@ void MarbleWidget::setSelection( const QRect& region )
     mDebug() << "Selection region: (" << tl.x() << ", " <<  tl.y() << ") (" 
              << br.x() << ", " << br.y() << ")" << endl;
 
-    AbstractProjection *proj = d->m_map->viewParams()->currentProjection();
-    GeoDataLatLonAltBox box  = proj->latLonAltBox( region, d->m_map->viewParams()->viewport() );
+    AbstractProjection *proj = viewport()->currentProjection();
+    GeoDataLatLonAltBox box  = proj->latLonAltBox( region, viewport() );
 
     // NOTE: coordinates as lon1, lat1, lon2, lat2 (or West, North, East, South)
     // as left/top, right/bottom rectangle.
@@ -1252,13 +1208,7 @@ QString MarbleWidget::distanceString() const
 
 void MarbleWidget::updateSun()
 {
-    // Update the sun shading.
-    //SunLocator  *sunLocator = d->m_model->sunLocator();
-
-    mDebug() << "MarbleWidget: Updating the sun shading map...";
-    d->m_model->update();
-    setNeedsUpdate();
-    //mDebug() << "Finished updating the sun shading map";
+    d->m_map->updateSun();
 }
 
 void MarbleWidget::centerSun()
@@ -1300,47 +1250,9 @@ void MarbleWidget::setInputEnabled( bool enabled )
     }
 }
 
-void MarbleWidget::setProxy( const QString& proxyHost, const quint16 proxyPort, const QString& user, const QString& password  )
-{
-    d->m_proxyHost = proxyHost;
-    d->m_proxyPort = proxyPort;
-    d->m_user = user;
-    d->m_password = password;
-
-    QNetworkProxy::ProxyType type = QNetworkProxy::HttpProxy;
-
-    // Make sure that no proxy is used for an empty string or the default value: 
-    if ( proxyHost.isEmpty() || proxyHost == "http://" )
-        type = QNetworkProxy::NoProxy;
-
-    QNetworkProxy proxy( type, d->m_proxyHost, d->m_proxyPort, d->m_user, d->m_password );
-    QNetworkProxy::setApplicationProxy( proxy );
-    mDebug() << "MarbleWidget::setProxy" << type << d->m_proxyHost << d->m_proxyPort << d->m_user << d->m_password;
-}
-
-QString MarbleWidget::proxyHost() const
-{
-    return d->m_proxyHost;
-}
-
-quint16 MarbleWidget::proxyPort() const
-{
-    return d->m_proxyPort;
-}
-
-QString MarbleWidget::user() const
-{
-    return d->m_user;
-}
-
-QString MarbleWidget::password() const
-{
-    return d->m_password;
-}
-
 QList<RenderPlugin *> MarbleWidget::renderPlugins() const
 {
-    return d->m_model->renderPlugins();
+    return d->m_map->renderPlugins();
 }
 
 void MarbleWidget::readPluginSettings( QSettings& settings )
@@ -1379,7 +1291,12 @@ void MarbleWidget::writePluginSettings( QSettings& settings ) const
 
 QList<AbstractFloatItem *> MarbleWidget::floatItems() const
 {
-    return d->m_model->floatItems();
+    return d->m_map->floatItems();
+}
+
+AbstractFloatItem * MarbleWidget::floatItem( const QString &nameId ) const
+{
+    return d->m_map->floatItem( nameId );
 }
 
 void MarbleWidget::changeEvent( QEvent * event )
@@ -1392,32 +1309,81 @@ void MarbleWidget::changeEvent( QEvent * event )
     QWidget::changeEvent(event);
 }
 
-void MarbleWidget::flyTo( const GeoDataLookAt &lookAt, FlyToMode mode )
+void MarbleWidget::flyTo( const GeoDataLookAt &newLookAt, FlyToMode mode )
 {
     if ( !d->m_animationsEnabled || mode == Instant ) {
-        d->m_map->flyTo( lookAt );
+        d->m_map->flyTo( newLookAt );
         d->repaint();
     }
     else {
-        GeoDataLookAt source = d->m_map->lookAt();
-        setViewContext( Marble::Animation );
-        ViewportParams *viewport = d->m_map->viewParams()->viewport();
-        d->m_physics->flyTo( source, lookAt, viewport, mode );
+        d->m_physics->flyTo( newLookAt, mode );
     }
 }
 
-void MarbleWidget::updateAnimation( const GeoDataLookAt &lookAt )
+void MarbleWidget::reloadMap()
 {
-    setViewContext( Marble::Animation );
-    d->m_map->flyTo( lookAt );
-    d->repaint();
+    d->m_map->reload();
 }
 
-void MarbleWidget::startStillMode()
+void MarbleWidget::downloadRegion( QString const & sourceDir, QVector<TileCoordsPyramid> const & pyramid )
 {
-    setViewContext( Marble::Still );
-    setNeedsUpdate();
-    d->repaint();
+    d->m_map->downloadRegion( sourceDir, pyramid );
+}
+
+GeoDataLookAt MarbleWidget::lookAt() const
+{
+    return d->m_map->lookAt();
+}
+
+void MarbleWidget::addBookmark( const GeoDataPlacemark &bookmark, const QString &folderName ) const
+{
+    d->m_model->bookmarkManager()->addBookmark( bookmark, folderName );
+}
+
+QString MarbleWidget::bookmarkFile() const
+{
+    return d->m_model->bookmarkManager()->bookmarkFile();
+}
+
+bool MarbleWidget::loadBookmarkFile( const QString &relativeFileName )
+{
+    return d->m_model->bookmarkManager()->loadFile( relativeFileName );
+}
+
+QVector<GeoDataFolder*> MarbleWidget::bookmarkFolders()
+{
+    return d->m_model->bookmarkManager()->folders();
+}
+
+void MarbleWidget::removeAllBookmarks()
+{
+    d->m_model->bookmarkManager()->removeAllBookmarks();
+}
+
+void MarbleWidget::addNewBookmarkFolder( const QString &name ) const
+{
+    d->m_model->bookmarkManager()->addNewBookmarkFolder( name );
+}
+
+RoutingLayer* MarbleWidget::routingLayer()
+{
+    return d->m_routingLayer;
+}
+
+void MarbleWidget::addGeoDataFile( const QString &filename )
+{
+    d->m_map->addGeoDataFile( filename );
+    //d->m_model->addGeoDataFile( filename );
+}
+
+void MarbleWidget::addGeoDataString( const QString &data, const QString &key )
+{
+    d->m_map->addGeoDataString( data, key );
+}
+
+void MarbleWidget::removeGeoData( const QString &key )
+{
+    d->m_map->removeGeoData( key );
 }
 
 }
