@@ -17,8 +17,6 @@
 
 #include "MergedLayerDecorator.h"
 
-#include <QtGui/QPainter>
-
 #include "SunLocator.h"
 #include "StackedTileLoader.h"
 #include "global.h"
@@ -32,10 +30,17 @@
 #include "StackedTile.h"
 #include "TileLoaderHelper.h"
 #include "Planet.h"
+#include "TileCreator.h"
+#include "TileCreatorDialog.h"
+#include "TileLoader.h"
+
+#include <QtGui/QPainter>
+#include <QtCore/QPointer>
+#include "TextureTile.h"
 
 using namespace Marble;
 
-MergedLayerDecorator::MergedLayerDecorator( StackedTileLoader * const tileLoader,
+MergedLayerDecorator::MergedLayerDecorator( TileLoader * const tileLoader,
                                             SunLocator* sunLocator )
     : m_tileLoader( tileLoader ),
       m_tile( 0 ),
@@ -57,6 +62,26 @@ void MergedLayerDecorator::initCityLights()
     m_cityLightsTheme = MapThemeManager::loadMapTheme( "earth/citylights/citylights.dgml" );
     if (m_cityLightsTheme) {
         QString cityLightsId = m_cityLightsTheme->head()->theme();
+        GeoSceneLayer* layer = m_cityLightsTheme->map()->layer( cityLightsId );
+        GeoSceneTexture *texture =
+            static_cast<GeoSceneTexture*>( layer->groundDataset() );
+
+        QString sourceDir = texture->sourceDir();
+        QString installMap = texture->installMap();
+        if ( !StackedTileLoader::baseTilesAvailable( layer ) ) {
+            TileCreator *tileCreator = new TileCreator(
+                                     sourceDir,
+                                     installMap,
+                                     "false" );
+
+            QPointer<TileCreatorDialog> tileCreatorDlg = new TileCreatorDialog( tileCreator, 0 );
+            tileCreatorDlg->setSummary( m_cityLightsTheme->head()->name(),
+                                        m_cityLightsTheme->head()->description() );
+            tileCreatorDlg->exec();
+            qDebug("Tile creation completed");
+            delete tileCreatorDlg;
+        }
+
         m_cityLightsTextureLayer = static_cast<GeoSceneTexture*>(
             m_cityLightsTheme->map()->layer( cityLightsId )->datasets().first() );
     }
@@ -67,12 +92,12 @@ MergedLayerDecorator::~MergedLayerDecorator()
     delete m_cityLightsTheme;
 }
 
-void MergedLayerDecorator::paint( const QString& themeId, GeoSceneDocument *mapTheme )
+void MergedLayerDecorator::paint( const QString& themeId )
 {
 //     QTime time;
 //     time.start();
     
-    if ( m_sunLocator->getShow() && mapTheme ) {
+    if ( m_sunLocator->getShow() ) {
 
         // Initialize citylights layer if it hasn't happened already
         if ( !m_cityLightsTheme ) {
@@ -97,12 +122,11 @@ bool MergedLayerDecorator::showTileId() const
     return m_showTileId;
 }
 
-StackedTile * MergedLayerDecorator::loadDataset( GeoSceneTexture *textureLayer )
+QImage MergedLayerDecorator::loadDataset()
 {
-    const TileId decorationTileId( textureLayer->sourceDir(), m_id.zoomLevel(), m_id.x(), m_id.y());
-    StackedTile * const tile = m_tileLoader->loadTile( decorationTileId, DownloadBrowse, true );
-    tile->setUsed( true );
-    return tile;
+    const TileId decorationTileId( m_cityLightsTextureLayer->sourceDir(), m_id.zoomLevel(), m_id.x(), m_id.y());
+    QImage image = *m_tileLoader->loadTile( decorationTileId, decorationTileId, DownloadBrowse )->image();
+    return image;
 }
 
 void MergedLayerDecorator::paintSunShading()
@@ -130,11 +154,7 @@ void MergedLayerDecorator::paintSunShading()
     //Don't use city lights on non-earth planets!
     if ( m_sunLocator->getCitylights() && m_sunLocator->planet()->id() == "earth" ) {
 
-        StackedTile * tile = loadDataset( m_cityLightsTextureLayer );
-        if ( tile->state() == StackedTile::TileEmpty )
-            return;
-
-        QImage * nighttile = tile->resultTile();
+        const QImage nighttile = loadDataset();
 
         for ( int cur_y = 0; cur_y < tileHeight; ++cur_y ) {
             qreal lat = lat_scale * ( m_id.y() * tileHeight + cur_y ) - 0.5*M_PI;
@@ -142,7 +162,7 @@ void MergedLayerDecorator::paintSunShading()
             qreal c = cos(lat)*cos( -DEG2RAD * m_sunLocator->getLat() );
 
             QRgb* scanline  = (QRgb*)m_tile->scanLine( cur_y );
-            QRgb* nscanline = (QRgb*)nighttile->scanLine( cur_y );
+            const QRgb* nscanline = (QRgb*)nighttile.scanLine( cur_y );
 
             qreal shade = 0;
             qreal lastShade = -10.0;
@@ -293,7 +313,7 @@ void MergedLayerDecorator::paintTileId( const QString& themeId )
     painter.drawRect( strokeWidth / 2, strokeWidth / 2, 
                       m_tile->width()  - strokeWidth,
                       m_tile->height() - strokeWidth );
-    QFont testFont( "Sans", 30, QFont::Bold );
+    QFont testFont( "Sans", 12 );
     QFontMetrics testFm( testFont );
     painter.setFont( testFont );
 
@@ -354,5 +374,3 @@ int MergedLayerDecorator::maxDivisor( int maximum, int fullLength )
     }
     return best;
 }
-
-#include "MergedLayerDecorator.moc"
