@@ -17,7 +17,6 @@
 #define MARBLE_MERGEDLAYERDECORATOR_H
 
 #include <QtGui/QImage>
-#include <QtCore/QObject>
 
 #include "TileId.h"
 #include "global.h"
@@ -31,42 +30,36 @@ class GeoSceneDocument;
 class GeoSceneTexture;
 class SunLocator;
 class StackedTile;
-class StackedTileLoader;
+class TileLoader;
 
-class MergedLayerDecorator : public QObject
+class MergedLayerDecorator
 {
-    Q_OBJECT
-	
  public:
-    MergedLayerDecorator( StackedTileLoader * const tileLoader, SunLocator* sunLocator );
+    MergedLayerDecorator( TileLoader * const tileLoader, SunLocator* sunLocator );
     virtual ~MergedLayerDecorator();
 
     // The Parameter themeId is only used for displaying the TileId,
     // which is a debugging feature, therefore at this point QString remains.
-    void paint( const QString& themeId, GeoSceneDocument *mapTheme = 0 );
-    void paintTileId(const QString& themeId);
-    
+    void paint( const QString& themeId );
+
     void setShowTileId(bool show);
     bool showTileId() const;
-	
+
     void setTile(QImage* tile);
     void setInfo( TileId const &id );
-	
- Q_SIGNALS:
-    void repaintMap();
-	
+
  private:
-    StackedTile * loadDataset( GeoSceneTexture *textureLayer );
+    QImage loadDataset();
     int maxDivisor( int maximum, int fullLength );
 
     void initCityLights();
 
     void paintSunShading();
-    void paintClouds();
-	
+    void paintTileId(const QString& themeId);
+
  protected:
     Q_DISABLE_COPY( MergedLayerDecorator )
-    StackedTileLoader * const m_tileLoader;
+    TileLoader * const m_tileLoader;
     QImage* m_tile;
     TileId m_id;
     SunLocator* m_sunLocator;
