@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -23,10 +23,12 @@
 
 #include "DeferredFlag.h"
 #include "GeoDataCoordinates.h"
+#include "GeoDataExtendedData.h"
 #include "MarbleDirs.h"
 #include "MarblePlacemarkModel.h"
 
 using namespace Marble;
+/* TRANSLATOR Marble::PlacemarkInfoDialog */
 
 PlacemarkInfoDialog::PlacemarkInfoDialog(const QPersistentModelIndex &index, QWidget *parent)
     : QDialog(parent), m_index(index)
@@ -59,6 +61,9 @@ void PlacemarkInfoDialog::showMessage( const QString& text )
 
 void PlacemarkInfoDialog::showContent()
 {
+    GeoDataPlacemark *placemark
+            = static_cast<GeoDataPlacemark*>(qVariantValue<GeoDataObject*>( m_index.model()->data(m_index, MarblePlacemarkModel::ObjectPointerRole ) ) );
+
     elevation_lbl->setVisible( true );
     elevation_val_lbl->setVisible( true );
     population_lbl->setText( tr("Population:") );
@@ -67,72 +72,56 @@ void PlacemarkInfoDialog::showContent()
     name_val_lbl->setText( "<H1><b>" + m_index.data().toString() + "</b></H1>" );
     altername_val_lbl->setText( "" );
     QString  rolestring;
-    switch ( m_index.data( MarblePlacemarkModel::GeoTypeRole ).toChar().toLatin1() ) {
-    case 'C':
+    const QString role = placemark->role();
+     if(role=="PPLC")
+        rolestring = tr("National Capital");
+    else if(role=="PPL")
+        rolestring = tr("City");
+    else if(role=="PPLA")
+        rolestring = tr("State Capital");
+    else if(role=="PPLA2")
+        rolestring = tr("County Capital");
+    else if(role=="PPLA3" || role=="PPLA4" )
         rolestring = tr("Capital");
-        break;
-    case 'B':
-        rolestring = tr("Capital");
-        break;
-    case 'R':
-        rolestring = tr("Regional Capital");
-        break;
-    case 'P':
+    else if(role=="PPLF" || role=="PPLG" || role=="PPLL" || role=="PPLQ" || role=="PPLR" || role=="PPLS" || role=="PPLW" )
+        rolestring = tr("Village");
+    else if(role=="P" || role=="M" )
         rolestring = tr("Location");
-        break;
-    case 'M':
-        rolestring = tr("Location");
-        break;
-    case 'H':
-        if ( m_index.data( MarblePlacemarkModel::PopularityRole ).toInt() > 0 )
+    else if(role=="H")
+    {    
+        if ( placemark->popularity() > 0 )
             rolestring = tr("Mountain");
         else
             rolestring = tr("Elevation extreme");
-        break;
-    case 'V':
-        rolestring = tr("Volcano");
-        break;
-    case 'W':
-        rolestring = tr("Shipwreck");
-        break;
-    case 'N':
-        rolestring = tr("City");
-        break;
-    case 'O':
-        rolestring = tr("Ocean");
-        break;
-    case 'S':
-        rolestring = tr("Nation");
-        break;
-    case 'K':
-        rolestring = tr("Continent");
-        break;
-    case 'A':
-        rolestring = tr("Astronomical observatory");
-        break;
-    case 'a':
-        rolestring = tr("Maria");
-        break;
-    case 'c':
-        rolestring = tr("Crater");
-        break;
-    case 'h':
-    case 'r':
-    case 'u':
-    case 'i':
-        rolestring = tr("Landing Site");
-        break;
-    default:
-        rolestring = tr("Other Place");
     }
-
+    else if(role=="V")
+        rolestring = tr("Volcano");
+    else if(role=="W")
+        rolestring = tr("Shipwreck");
+    else if(role=="O")
+        rolestring = tr("Ocean");
+    else if(role=="S")
+        rolestring = tr("Nation");
+    else if(role=="K")
+        rolestring = tr("Continent");
+    else if(role=="A")
+        rolestring = tr("Astronomical observatory");
+    else if(role=="a")
+        rolestring = tr("Maria");
+    else if(role=="c")
+        rolestring = tr("Crater");
+    else if(role=="h" || role=="r" || role=="u" || role=="i")
+        rolestring = tr("Landing Site");
+    else 
+	rolestring = tr("Other Place");
+	
     role_val_lbl->setText( rolestring );
 
     m_flagcreator = new DeferredFlag( this );
-    requestFlag( m_index.data( MarblePlacemarkModel::CountryCodeRole ).toString() );
+    requestFlag( placemark->countryCode() );
 
 
-    const QString description = m_index.data( MarblePlacemarkModel::DescriptionRole ).toString();
+    const QString description = placemark->description();
 
     description_val_browser->setEnabled( false );
     if ( !description.isEmpty() )
@@ -140,12 +129,16 @@ void PlacemarkInfoDialog::showContent()
         description_val_browser->setEnabled( true );
         description_val_browser->setHtml( description );
     }
-    coordinates_val_lbl->setText( m_index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>().toString() );
-    country_val_lbl->setText( m_index.data( MarblePlacemarkModel::CountryCodeRole ).toString() );
+    coordinates_val_lbl->setText( placemark->coordinate().toString() );
+    country_val_lbl->setText( placemark->countryCode() );
+    QString gmt = QString( "%1" ).arg( placemark->extendedData().value("gmt").value().toInt()/( double ) 100, 0, 'f', 1 );
+    QString dst = QString( "%1" ).arg( ( placemark->extendedData().value("gmt").value().toInt() + placemark->extendedData().value("dst").value().toInt() )/( double ) 100, 0, 'f', 1 );
+    gmtdst_val_lbl->setText( gmt + " / " + dst );
+    state_val_lbl->setText( placemark->state() );
 
-    const qint64 population = m_index.data( MarblePlacemarkModel::PopulationRole ).toLongLong();
-    const qreal area = m_index.data( MarblePlacemarkModel::AreaRole ).toDouble();
-    const qreal altitude = m_index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>().altitude();
+    const qint64 population = placemark->population();
+    const qreal area = placemark->area();
+    const qreal altitude = placemark->coordinate().altitude();
 
     area_lbl->setText( tr("Area:") );
     if ( area < 10000000 )
@@ -163,8 +156,6 @@ void PlacemarkInfoDialog::showContent()
     elevation_val_lbl->setText( tr("%1 m").arg( QLocale::system().toString( altitude ) ) );
     diameter_val_lbl->setText( tr("%1 km").arg( QLocale::system().toString( population / 1000.0, 'g', 4 ) ) );
 
-    const QChar role = m_index.data( MarblePlacemarkModel::GeoTypeRole ).toChar();
-
     country_lbl->setVisible( true );
     country_val_lbl->setVisible( true );
     elevation_lbl->setVisible( true );
@@ -175,57 +166,87 @@ void PlacemarkInfoDialog::showContent()
     population_val_lbl->setVisible( true );
     diameter_lbl->setVisible( false );
     diameter_val_lbl->setVisible( false );
-    timezone_lbl->setVisible( false );
-    timezone_val_lbl->setVisible( false );
+    gmtdst_lbl->setVisible( true );
+    gmtdst_val_lbl->setVisible( true );
+    state_lbl->setVisible( true );
+    state_val_lbl->setVisible( true );
 
     if ( altitude <= 0 )
         elevation_val_lbl->setText( tr("-") );
 
-    if (    role == 'O' ||  role == 'o' || role == 'v' || role == 'h' || role == 'u'
-         || role == 'i' || role == 'r' || role == 'a' || role == 'c' || role == 'm' ) {
+    if (    role == "O" ||  role == "o" || role == "v" || role == "h" || role == "u"
+         || role == "i" || role == "r" || role == "a" || role == "c" || role == "m" ) {
         population_val_lbl->setVisible( false );
         population_lbl->setVisible( false );
         country_lbl->setVisible( false );
         country_val_lbl->setVisible( false );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
 
-    if ( role == 'A' ) {
+    if ( role == "A" ) {
         population_val_lbl->setVisible( false );
         population_lbl->setVisible( false );
         country_lbl->setVisible( false );
         country_val_lbl->setVisible( false );
         elevation_val_lbl->setVisible( false );
         elevation_lbl->setVisible( false );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
 
-    if ( role == 'K' )
+    if ( role == "K" )
     {
         country_lbl->setVisible( false );
         country_val_lbl->setVisible( false );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
-    if ( (role == 'a' || role == 'c' || role == 'm') && m_index.data( MarblePlacemarkModel::PopularityRole ).toInt() > 0) {
+    if ( (role == "a" || role == "c" || role == "m") && placemark->popularity() > 0) {
         diameter_lbl->setVisible( true );
         diameter_val_lbl->setVisible( true );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
 
-    if ( role == 'H' || role == 'V' || role == 'W') {
+    if ( role == "H" || role == "V" || role == "W") {
         population_val_lbl->setVisible( false );
         population_lbl->setVisible( false );
         area_val_lbl->setVisible( false );
         area_lbl->setVisible( false );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
-    else if ( role == 'P' || role == 'M' ) {
+    else if ( role == "P" || role == "M" ) {
         population_val_lbl->setVisible( false );
         population_lbl->setVisible( false );
         elevation_val_lbl->setVisible( false );
         elevation_lbl->setVisible( false );
         area_val_lbl->setVisible( false );
         area_lbl->setVisible( false );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
-    else if ( role == 'O' || role == 'K' || role == 'S'  )
+    else if ( role == "O" || role == "K" || role == "S"  )
     {
         elevation_lbl->setVisible( false );
         elevation_val_lbl->setVisible( false );
+        gmtdst_lbl->setVisible( false );
+        gmtdst_val_lbl->setVisible( false );
+        state_lbl->setVisible( false );
+        state_val_lbl->setVisible( false );
     }
     else{
         area_val_lbl->setVisible( false );
