@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -38,11 +38,13 @@
 
 using namespace Marble;
 
-PlacemarkLayout::PlacemarkLayout( QObject* parent )
+PlacemarkLayout::PlacemarkLayout( const QAbstractItemModel  *placemarkModel,
+                                  const QItemSelectionModel *selectionModel,
+                                  QObject* parent )
     : QObject( parent ),
+      m_placemarkModel( placemarkModel ),
+      m_selectionModel( selectionModel ),
       m_placemarkPainter( 0 ),
-      m_placemarkModel( 0 ),
-      m_selectionModel( 0 ),
       m_maxLabelHeight( 0 ),
       m_styleResetRequested( true )
 {
@@ -145,37 +147,34 @@ QVector<QModelIndex> PlacemarkLayout::whichPlacemarkAt( const QPoint& curpos )
     return ret;
 }
 
-PlacemarkPainter* PlacemarkLayout::placemarkPainter() const
-{ 
-    return m_placemarkPainter; 
-}
-
-int PlacemarkLayout::maxLabelHeight( const QAbstractItemModel* model,
-                                     const QItemSelectionModel* selectionModel ) const
+int PlacemarkLayout::maxLabelHeight() const
 {
     mDebug() << "Detecting maxLabelHeight ...";
 
     int maxLabelHeight = 0;
 
-    const QModelIndexList selectedIndexes = selectionModel->selection().indexes();
+    const QModelIndexList selectedIndexes = m_selectionModel->selection().indexes();
 
     for ( int i = 0; i < selectedIndexes.count(); ++i ) {
         const QModelIndex index = selectedIndexes.at( i );
-        GeoDataStyle* style = qvariant_cast<GeoDataStyle*>( index.data( MarblePlacemarkModel::StyleRole ) );
+        GeoDataPlacemark *placemark = dynamic_cast<GeoDataPlacemark*>(qvariant_cast<GeoDataObject*>(index.data( MarblePlacemarkModel::ObjectPointerRole ) ));
+        GeoDataStyle* style = placemark->style();
         QFont labelFont = style->labelStyle().font();
         int textHeight = QFontMetrics( labelFont ).height();
         if ( textHeight > maxLabelHeight )
             maxLabelHeight = textHeight; 
     }
 
-    for ( int i = 0; i < model->rowCount(); ++i ) {
-        QModelIndex index = model->index( i, 0 );
-
-        GeoDataStyle* style = qvariant_cast<GeoDataStyle*>( index.data( MarblePlacemarkModel::StyleRole ) );
-        QFont labelFont = style->labelStyle().font();
-        int textHeight = QFontMetrics( labelFont ).height();
-        if ( textHeight > maxLabelHeight ) 
-            maxLabelHeight = textHeight; 
+    for ( int i = 0; i < m_placemarkModel->rowCount(); ++i ) {
+        QModelIndex index = m_placemarkModel->index( i, 0 );
+        GeoDataPlacemark *placemark = dynamic_cast<GeoDataPlacemark*>(qvariant_cast<GeoDataObject*>(index.data( MarblePlacemarkModel::ObjectPointerRole ) ));
+        if ( placemark ) {
+            GeoDataStyle* style = placemark->style();
+            QFont labelFont = style->labelStyle().font();
+            int textHeight = QFontMetrics( labelFont ).height();
+            if ( textHeight > maxLabelHeight )
+                maxLabelHeight = textHeight;
+        }
     }
 
     //mDebug() <<"Detected maxLabelHeight: " << maxLabelHeight;
@@ -183,33 +182,23 @@ int PlacemarkLayout::maxLabelHeight( const QAbstractItemModel* model,
 }
 
 void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
-                                        ViewParams *viewParams,
-                                        bool firstTime )
+                                        ViewParams *viewParams )
 {
-    paintPlaceFolder( painter, viewParams, model(), selectionModel(), firstTime );
-}
-
-void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
-                                        ViewParams *viewParams,
-                                        const QAbstractItemModel  *model,
-                                        const QItemSelectionModel *selectionModel,
-                                        bool firstTime )
-{
-    Q_UNUSED( firstTime );
-
-    if( 0 == selectionModel )
+    if ( m_placemarkModel->rowCount() <= 0 )
         return;
-    if( 0 == model )
-        return;
-        
+
     // const int imgwidth  = viewParams->canvasImage()->width();
     const int imgheight = viewParams->canvasImage()->height();
 
     if ( m_styleResetRequested ) {
         m_styleResetRequested = false;
         styleReset();
-        
-        m_maxLabelHeight = maxLabelHeight( model, selectionModel );
+
+        m_maxLabelHeight = maxLabelHeight();
+    }
+
+    if ( m_maxLabelHeight == 0 ) {
+        return;
     }
     const int   secnumber         = imgheight / m_maxLabelHeight + 1;
 
@@ -246,18 +235,17 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
      * First handle the selected placemarks, as they have the highest priority.
      */
 
-    const QModelIndexList selectedIndexes = selectionModel->selection().indexes();
+    const QModelIndexList selectedIndexes = m_selectionModel->selection().indexes();
 
     for ( int i = 0; i < selectedIndexes.count(); ++i ) {
         const QModelIndex index = selectedIndexes.at( i );
-
-        GeoDataGeometry geometry
-                = qvariant_cast<GeoDataGeometry>( index.data( MarblePlacemarkModel::GeometryRole ) );
-        if( geometry.geometryId() != GeoDataPointId ) {
+        GeoDataPlacemark *placemark = dynamic_cast<GeoDataPlacemark*>(qvariant_cast<GeoDataObject*>(index.data( MarblePlacemarkModel::ObjectPointerRole ) ));
+        GeoDataGeometry *geometry = placemark->geometry();
+        if( !dynamic_cast<GeoDataPoint*>(geometry) ) {
             continue;
         }
 
-        GeoDataCoordinates geopoint = qvariant_cast<GeoDataCoordinates>( index.data( MarblePlacemarkModel::CoordinateRole ) );
+        GeoDataCoordinates geopoint = placemark->coordinate();
 
         if ( !latLonAltBox.contains( geopoint ) ||
              ! viewParams->currentProjection()->screenCoordinates( geopoint, viewParams->viewport(), x, y ))
@@ -273,7 +261,7 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
         // Find the corresponding visible placemark
         VisiblePlacemark *mark = m_visiblePlacemarks.value( index );
 
-        GeoDataStyle* style = qvariant_cast<GeoDataStyle*>( index.data( MarblePlacemarkModel::StyleRole ) );
+        GeoDataStyle* style = placemark->style();
 
         // Specify font properties
         if ( mark ) {
@@ -283,7 +271,7 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
             QFont labelFont = style->labelStyle().font();
             labelFont.setWeight( 75 ); // Needed to calculate the correct pixmap size; 
 
-            textWidth = ( QFontMetrics( labelFont ).width( index.data( Qt::DisplayRole ).toString() )
+            textWidth = ( QFontMetrics( labelFont ).width( placemark->name() )
                   + (int)( 2 * s_labelOutlineWidth ) );
         }
 
@@ -334,42 +322,37 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
     /**
      * Now handle all other placemarks...
      */
-    const QModelIndex firstIndex = model->index( 0, 0 );
-    const int firstPopularity = firstIndex.data( MarblePlacemarkModel::PopularityRole ).toInt();
-    const bool  noFilter = ( firstPopularity == 0 
-                             || ( firstPopularity != 0
-                             && firstIndex.data( MarblePlacemarkModel::GeoTypeRole ).toChar().isNull() ) ) 
-                           ? true : false;
-    const QItemSelection selection = selectionModel->selection();
+    const QItemSelection selection = m_selectionModel->selection();
 
-    const int rowCount = model->rowCount();
+    const int rowCount = m_placemarkModel->rowCount();
 
     for ( int i = 0; i != rowCount; ++i )
     {
-        const QModelIndex& index = model->index( i, 0 );
+        const QModelIndex& index = m_placemarkModel->index( i, 0 );
         if( !index.isValid() ) {
             mDebug() << "invalid index!!!";
             continue;
         }
 
-        GeoDataGeometry geometry
-                = qvariant_cast<GeoDataGeometry>( index.data( MarblePlacemarkModel::GeometryRole ) );
-        if( geometry.geometryId() != GeoDataPointId ) {
+        GeoDataPlacemark *placemark = dynamic_cast<GeoDataPlacemark*>(qvariant_cast<GeoDataObject*>(index.data( MarblePlacemarkModel::ObjectPointerRole ) ));
+        GeoDataGeometry *geometry = placemark->geometry();
+        if( !dynamic_cast<GeoDataPoint*>(geometry) ) {
             continue;
         }
 
-        int popularityIndex = index.data( MarblePlacemarkModel::PopularityIndexRole ).toInt();
+        int popularityIndex = placemark->popularityIndex();
 
+        
         if ( popularityIndex < 1 ) {
             break;
         }
 
         // Skip the places that are too small.
-        if ( !noFilter && m_weightfilter.at( popularityIndex ) > viewParams->radius() ) {
+        if ( m_weightfilter.at( popularityIndex ) > viewParams->radius() ) {
             break;
         }
 
-        GeoDataCoordinates geopoint = qvariant_cast<GeoDataCoordinates>( index.data( MarblePlacemarkModel::CoordinateRole ) );
+        GeoDataCoordinates geopoint = placemark->coordinate();
 
         if ( !latLonAltBox.contains( geopoint ) ||
              ! viewParams->currentProjection()->screenCoordinates( geopoint, viewParams->viewport(), x, y ))
@@ -378,7 +361,7 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
                 continue;
             }
 
-        const int visualCategory  = index.data( MarblePlacemarkModel::VisualCategoryRole ).toInt();
+        const int visualCategory  = placemark->visualCategory();
 
         // Skip city marks if we're not showing cities.
         if ( !showCities
@@ -429,7 +412,7 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
 
         // Find the corresponding visible placemark
         VisiblePlacemark *mark = m_visiblePlacemarks.value( index );
-        GeoDataStyle* style = qvariant_cast<GeoDataStyle*>( index.data( MarblePlacemarkModel::StyleRole ) );
+        GeoDataStyle* style = placemark->style();
 
         // Specify font properties
         if ( mark ) {
@@ -437,7 +420,7 @@ void PlacemarkLayout::paintPlaceFolder( QPainter   *painter,
         }
         else {
             QFont labelFont = style->labelStyle().font();
-            textWidth = ( QFontMetrics( labelFont ).width( index.data( Qt::DisplayRole ).toString() ) );
+            textWidth = ( QFontMetrics( labelFont ).width( placemark->name() ) );
         }
 
         // Choose Section
@@ -583,24 +566,4 @@ int PlacemarkLayout::placemarksOnScreenLimit() const
     return 100;
 }
 
-void PlacemarkLayout::setModel( QAbstractItemModel *model )
-{
-    m_placemarkModel = model;
-}
-
-QAbstractItemModel * PlacemarkLayout::model() const
-{
-    return m_placemarkModel;
-}
-
-void PlacemarkLayout::setSelectionModel( QItemSelectionModel * selectionModel )
-{
-    m_selectionModel = selectionModel;
-}
-
-QItemSelectionModel * PlacemarkLayout::selectionModel() const
-{
-    return m_selectionModel;
-}
-
 #include "PlacemarkLayout.moc"
