@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -112,6 +112,17 @@ bool AbstractProjection::screenCoordinates( const GeoDataCoordinates &geopoint,
     return screenCoordinates( geopoint, viewport, x, y, globeHidesPoint );
 }
 
+bool AbstractProjection::screenCoordinates( const GeoDataCoordinates &geopoint,
+                                            const ViewportParams *viewport,
+                                            QPointF &screenpoint )
+{
+    bool visible;
+    qreal x(0), y(0);
+    visible = screenCoordinates( geopoint, viewport, x, y );
+    screenpoint = QPointF( x,y );
+    return visible;
+}
+
 bool AbstractProjection::screenCoordinates( const GeoDataCoordinates &coordinates,
                                     const ViewportParams *viewport,
                                     qreal *x, qreal &y, int &pointRepeatNum, bool &globeHidesPoint )
