@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -39,7 +39,6 @@ class MarbleDataFacade;
 class MARBLE_EXPORT RenderPlugin : public QObject, public RenderPluginInterface
 {
     Q_OBJECT
-    Q_INTERFACES( Marble::RenderPluginInterface )
 
  public:
     /**
@@ -53,6 +52,11 @@ class MARBLE_EXPORT RenderPlugin : public QObject, public RenderPluginInterface
         BackendTypes                     // a QStringList
     };
 
+    enum RenderType {
+        Unknown,
+        Online
+    };
+
     RenderPlugin();
     virtual ~RenderPlugin();
 
@@ -109,6 +113,14 @@ class MARBLE_EXPORT RenderPlugin : public QObject, public RenderPluginInterface
      */
     virtual void setSettings( QHash<QString,QVariant> settings );
 
+    /**
+     * Function for returning the type of plugin this is for.
+     * This affects where in the menu tree the action() is placed.
+     *
+     * @return: The type of render plugin this is.
+     */
+    virtual RenderType renderType() const;
+
  public Q_SLOTS:
     void    setEnabled( bool enabled );
     void    setVisible( bool visible );
@@ -146,7 +158,7 @@ class MARBLE_EXPORT RenderPlugin : public QObject, public RenderPluginInterface
 };
 
 #define MARBLE_PLUGIN(T) public:\
-    virtual RenderPlugin* pluginInstance() { return new T(); };
+    virtual RenderPlugin* newInstance() { return new T(); }
 }
 
 #endif
